@@ -31,7 +31,7 @@
 #include <mbase/resources.h>
 
 
-const char* APROJECT_VERSION = "1.0.2";
+const char* APROJECT_VERSION = "1.2.1";
 
 namespace Project
 {
@@ -258,6 +258,8 @@ void VideoMD::SerializeImpl(Archieve& ar)
         ArchieveStackFrame asf(ar, "DVDTransData");
         ar("DVDDims", transDat.dd   )
           ("VRate",   transDat.vRate);
+        if( CanSrl(ar, 3) )
+            ar("CustomFFmpegOptions" , transDat.ctmFFOpt);
     }
     {
         ArchieveStackFrame asf(ar, "SubtitleData");
@@ -344,6 +346,8 @@ void SaveFormattedUTF8Xml(xmlpp::Document& doc, const Glib::ustring& filename)
     doc.write_to_file_formatted(filename, "UTF-8");
 }
 
+const int PRJ_VERSION = 2;
+
 bool ADatabase::SaveWithFnr(ArchieveFnr afnr)
 {
     ASSERT( !isOut );
@@ -351,13 +355,11 @@ bool ADatabase::SaveWithFnr(ArchieveFnr afnr)
         return false;
     IndexOrderArrs();
 
-    xmlpp::Document doc;
-    xmlpp::Element* root_node = doc.create_root_node("AProject");
-    root_node->set_attribute("Version", APROJECT_VERSION);
+    XMLSave xs("AProject");
+    xmlpp::Element* root_node = xs.rootNode;
+    root_node->set_attribute("BmdVersion", APROJECT_VERSION);
     root_node->add_child_comment("This document is for " APROGRAM_PRINTABLE_NAME " program");
-
-    DoSaveArchieve(root_node, afnr);
-    SaveFormattedUTF8Xml(doc, ConvertPathFromUtf8(prjFName));
+    SaveXS(xs, afnr, PRJ_VERSION, ConvertPathFromUtf8(prjFName));
     return true;
 }
 
@@ -376,14 +378,19 @@ void ADatabase::LoadWithFnr(const std::string& fname, ArchieveFnr afnr,
 // ThemeDirList
 // 
 
-ThemeDirList::ThemeDirList()
+void AddSrcDirs(Str::List& dirs, const char* dname)
 {
     // директории по умолчанию
-    std::string user_frames = GetConfigDir() + "/frames";
+    std::string user_frames = fs::path(GetConfigDir()) / dname / fs::to_str;
     if( CreateDirsQuiet(user_frames) )
-        AddDir(user_frames);
+        dirs.push_back(user_frames);
 
-    AddDir(DataDirPath("frames"));
+    dirs.push_back(DataDirPath(dname));
+}
+
+ThemeDirList::ThemeDirList()
+{
+    AddSrcDirs(*this, "frames");
 }
 
 static bool IsFrameDir(const fs::path& f_dir)
@@ -404,6 +411,11 @@ fs::path FindThemePath(const std::string& theme_name)
     return fs::path();
 }
 
+void MakeUnique(Str::List& t_lst)
+{
+    t_lst.resize(std::unique(t_lst.begin(), t_lst.end()) - t_lst.begin());
+}
+
 void GetThemeList(Str::List& t_lst)
 {
     t_lst.clear();
@@ -421,7 +433,7 @@ void GetThemeList(Str::List& t_lst)
 
     // * сортируем и удаляем дублирование
     std::sort(t_lst.begin(), t_lst.end());
-    t_lst.resize(std::unique(t_lst.begin(), t_lst.end()) - t_lst.begin());
+    MakeUnique(t_lst);
 }
 
 bool IsPALProject()
