@@ -22,6 +22,7 @@
 #ifndef __MGUI_AUTHOR_FFMPEG_H__
 #define __MGUI_AUTHOR_FFMPEG_H__
 
+#include <mgui/project/video.h> // AutoSrcData
 #include <mgui/sdk/ioblock.h>
 
 #include <mdemux/trackbuf.h>
@@ -36,9 +37,9 @@ struct DVDTransData;
 
 struct AutoDVDTransData
 {
-     bool  is4_3;
-    Point  srcAspect;
-      int  audioNum; // для <= 1 ffmpeg сам решит
+       bool  is4_3;
+AutoSrcData  asd;   
+        int  threadsCnt;
 
     AutoDVDTransData(bool is4_3_);
 };
@@ -46,9 +47,19 @@ struct AutoDVDTransData
 std::string FFmpegToDVDArgs(const std::string& out_fname, const AutoDVDTransData& atd,
                             bool is_pal, const DVDTransData& td);
 std::string FFmpegToDVDArgs(const std::string& out_fname, bool is_4_3, bool is_pal);
+
 // для меню
+struct AudioArgInput
+{
+    std::string  fName;
+         double  shift;
+
+    AudioArgInput(const std::string& a_fname = std::string(), double a_shift = 0.)
+        :fName(a_fname), shift(a_shift) {}
+};
+
 std::string FFmpegPostArgs(const std::string& out_fname, bool is_4_3, bool is_pal, 
-                           const std::string& a_fname = std::string(), double a_shift = 0.);
+                           const AudioArgInput& aai = AudioArgInput());
 
 // полная команда для транскодирования
 std::string FFmpegToDVDTranscode(const std::string& src_fname, const std::string& dst_fname,
@@ -92,11 +103,15 @@ struct TripleVersion
 
 struct FFmpegVersion
 {
+    TripleVersion avcodec;
+    TripleVersion avformat;
     TripleVersion avfilter;
 };
 
+TripleVersion FindAVFilterVersion(const std::string& conts);
+
 // conts - вывод ffmpeg -formats
-FFmpegVersion TestFFmpegForDVDEncoding(const std::string& conts);
+void TestFFmpegForDVDEncoding(const std::string& conts);
 // проверка возможности кодировать совместимый с DVD материал с помощью ffmpeg
 FFmpegVersion CheckFFDVDEncoding();
 
@@ -104,6 +119,16 @@ TripleVersion FindVersion(const std::string& conts, const re::pattern& ver_pat,
                           const char* app_name, const char* target_name = 0);
 bool IsVersionGE(const TripleVersion& big_v, const TripleVersion& v);
 
+Gtk::TextView& PrintCmdToDetails(const std::string& cmd);
+ReadReadyFnr DetailsAppender(const std::string& print_cmd, 
+                             const ReadReadyFnr& add_fnr = ReadReadyFnr(),
+                             const std::string& prefix = std::string());
+
+void RunSpumux(const std::string& xml_fname, const std::string& src_fname, const std::string& dst_fname,
+               int stream_id = 0, const char* dir = 0);
+
+const char* AVCnvBin();
+
 } // namespace Project
 
 #endif // #ifndef __MGUI_AUTHOR_FFMPEG_H__
