@@ -34,15 +34,15 @@
 #include <mgui/sdk/browser.h>   // VideoStart
 #include <mgui/sdk/ioblock.h>
 #include <mgui/sdk/textview.h>  // AppendCommandText()
-#include <mgui/ffviewer.h>
+#include <mgui/prefs.h>
 
 #include <mbase/project/table.h>
+#include <mbase/project/theme.h> // FindThemePath()
 
 #include <mlib/filesystem.h>
 #include <mlib/read_stream.h> // ReadAllStream()
 #include <mlib/sdk/system.h> // GetClockTime()
 #include <mlib/gettext.h>
-#include <mlib/regex.h>
 
 #include <boost/lexical_cast.hpp>
 
@@ -55,13 +55,48 @@ void IteratePendingEvents()
 namespace Project 
 {
 
+struct ShiftPCB: public PixCanvasBuf
+{
+    Point shift;
+    
+ virtual RefPtr<Gdk::Pixbuf> Canvas();
+};
+
+RefPtr<Gdk::Pixbuf> ShiftPCB::Canvas()
+{
+    RefPtr<Gdk::Pixbuf> res;
+    if( canvPix )
+    {
+        Point sz = PixbufSize(canvPix);
+        Rect rct(shift, sz);
+        ASSERT( rct.IsValid() );
+        res = MakeSubPixbuf(canvPix, Rect(shift, sz));
+    }
+    return res;
+}
+
+static const char* SubPicTag(bool not_letterbox16_9)
+{
+    return not_letterbox16_9 ? AUTHOR_TAG : AUTHOR_LB_TAG;
+}
+
 PixCanvasBuf& GetTaggedPCB(Menu mn, const char* tag)
 {
-    PixCanvasBuf& pcb = mn->GetData<PixCanvasBuf>(tag);
+    bool is_lb_tag = strcmp(AUTHOR_LB_TAG, tag) == 0;
+
+    PixCanvasBuf& pcb = !is_lb_tag ? mn->GetData<PixCanvasBuf>(tag) : mn->GetData<ShiftPCB>(tag) ;
     if( !pcb.Canvas() )
     {
         Point sz(mn->Params().Size());
-        pcb.Set(CreatePixbuf(sz), Planed::Transition(Rect0Sz(sz), sz));
+        Rect plc(Rect0Sz(sz));
+        if( is_lb_tag )
+        {
+            // сжимаем 4:3 до 16:9 = *3/4
+            plc.btm = Round(3./4. * sz.y);
+            static_cast<ShiftPCB&>(pcb).shift = Point(0, (sz.y - plc.btm)/2);
+        }
+        pcb.Set(CreatePixbuf(sz), Planed::Transition(plc, sz));
+
         pcb.DataTag() = tag;
     }
     return pcb;
@@ -97,8 +132,8 @@ class SPRenderVis: public CommonMenuRVis
 {
     typedef CommonMenuRVis MyParent;
     public:
-                  SPRenderVis(RectListRgn& r_lst, xmlpp::Element* spu_node)
-                    : MyParent(AUTHOR_TAG, r_lst), isSelect(false), spuNode(spu_node) {}
+                  SPRenderVis(RectListRgn& r_lst, xmlpp::Element* spu_node, bool not_letterbox16_9)
+                    : MyParent(SubPicTag(not_letterbox16_9), r_lst), isSelect(false), spuNode(spu_node) {}
 
             void  SetSelect(bool is_select) { isSelect = is_select; }
 
@@ -160,16 +195,21 @@ class SubtitleWrapper
     }
 };
 
-static void ScriptButton(xmlpp::Element* spu_node, bool is_select, const Rect& plc)
+static void ScriptButton(xmlpp::Element* spu_node, bool is_select, const Rect& plc,
+                         CanvasBuf* cnv_buf)
 {
     if( !is_select )
     {
         xmlpp::Element* btn_node = spu_node->add_child("button");
         // :KLUDGE: должны быть четными
-        btn_node->set_attribute("x0", boost::lexical_cast<std::string>(plc.lft));
-        btn_node->set_attribute("y0", boost::lexical_cast<std::string>(plc.top));
-        btn_node->set_attribute("x1", boost::lexical_cast<std::string>(plc.rgt));
-        btn_node->set_attribute("y1", boost::lexical_cast<std::string>(plc.btm));
+        Rect rct(plc);
+        // для letterbox & 16:9
+        if( ShiftPCB* s_pcb = dynamic_cast<ShiftPCB*>(cnv_buf) )
+            rct += s_pcb->shift;
+        btn_node->set_attribute("x0", boost::lexical_cast<std::string>(rct.lft));
+        btn_node->set_attribute("y0", boost::lexical_cast<std::string>(rct.top));
+        btn_node->set_attribute("x1", boost::lexical_cast<std::string>(rct.rgt));
+        btn_node->set_attribute("y1", boost::lexical_cast<std::string>(rct.btm));
     }
 }
 
@@ -220,16 +260,22 @@ void SPRenderVis::Visit(TextObj& t_obj)
         Rect plc = CalcRelPlacement(t_obj.Placement());
         DiscreteByAlpha(MakeSubPixbuf(drw->Canvas(), plc), clr);
 
-        ScriptButton(spuNode, isSelect, plc);
+        ScriptButton(spuNode, isSelect, plc, cnvBuf);
     }
 }
 
+static RefPtr<Gdk::Pixbuf> CopyScale(RefPtr<Gdk::Pixbuf> src, const Point& sz)
+{
+    RefPtr<Gdk::Pixbuf> pix = CreatePixbuf(sz);
+    RGBA::Scale(pix, src);
+    return pix;
+}
+
 void SPRenderVis::Visit(FrameThemeObj& fto)
 {
     std::string targ_str;    
     if( HasButtonLink(fto, targ_str) )
     {
-        const Editor::ThemeData& td = Editor::ThemeCache::GetTheme(fto.Theme());
         // как и в случае текста, для субтитров DVD главное - не перебрать ограничение
         // кол-ва используемых цветов (<=16); поэтому подгоняем рамку по месту, а
         // не наоборот (чтоб обойти без финального скалирования) 
@@ -239,17 +285,34 @@ void SPRenderVis::Visit(FrameThemeObj& fto)
         uint clr = GetSPColor(fto, isSelect).ToUint();
         obj_pix->fill(clr);
 
-        RefPtr<Gdk::Pixbuf> vf_pix = CreatePixbuf(sz);
-        RGBA::Scale(vf_pix, td.vFrameImg);
+        RefPtr<Gdk::Pixbuf> alpha_pix;
+        if( IsIconTheme(fto) )
+            alpha_pix = GetThemeIcon(fto, sz);
+        else
+        {
+            const Editor::ThemeData& td = Editor::ThemeCache::GetTheme(fto.ThemeName());
+            if( fto.hlBorder )
+            {
+                fs::path brd_fname = Project::FindThemePath(fto.ThemeName()) / "highlight_border.png";
+                if( fs::exists(brd_fname) )
+                    alpha_pix = CopyScale(Gdk::Pixbuf::create_from_file(brd_fname.string()), sz);
+                else
+                {    
+                    alpha_pix = CopyScale(td.frameImg, sz);
+                    RGBA::ApplyPicture(alpha_pix, CopyScale(td.vFrameImg, sz), RGBA::potDELETE_ALPHA);
+                }
+            }
+            else
+                alpha_pix = CopyScale(td.vFrameImg, sz);
+        }
 
-        RGBA::CopyAlphaComposite(obj_pix, vf_pix, true);
+        RGBA::CopyAlphaComposite(obj_pix, alpha_pix, true);
         DiscreteByAlpha(obj_pix, clr);
 
-        //drw->CompositePixbuf(obj_pix, plc);
         RGBA::RgnPixelDrawer::DrwFunctor drw_fnr = bb::bind(&RGBA::CopyArea, drw->Canvas(), obj_pix, plc, _1);
         drw->DrawWithFunctor(plc, drw_fnr);
 
-        ScriptButton(spuNode, isSelect, plc);
+        ScriptButton(spuNode, isSelect, plc, cnvBuf);
     }
 }
 
@@ -270,11 +333,6 @@ void InitTextForTaggedPCB(Menu mn, const char* tag)
     GetMenuRegion(mn).Accept(titv);
 }
 
-void InitAuthorData(Menu mn)
-{
-    InitTextForTaggedPCB(mn, AUTHOR_TAG);
-}
-
 static void IterateAuthoringEvents()
 {
     IteratePendingEvents();
@@ -296,30 +354,31 @@ static void PulseRenderProgress()
     IterateAuthoringEvents();
 }
 
-bool RenderSubPictures(const std::string& out_dir, Menu mn, int i, 
-                       std::iostream& menu_list)
+void MakeSubpictures(Menu mn, bool not_letterbox16_9, const std::string& mn_dir)
 {
-    InitAuthorData(mn);
+    const char* data_tag = SubPicTag(not_letterbox16_9);
+    InitTextForTaggedPCB(mn, data_tag);
 
     MenuRegion& m_rgn   = GetMenuRegion(mn);
-    CanvasBuf&  cnv_buf = GetAuthorPCB(mn);
-    RefPtr<Gdk::Pixbuf> cnv_pix = cnv_buf.FramePixbuf();
-    // :WARN: правильно ли обращаемся (на питоне) к файловой системе не в utf8?
-    std::string mn_dir = MenuAuthorDir(mn, i, false);
-    menu_list << "'''" << mn_dir << "''',\n"; 
-    mn_dir = AppendPath(out_dir, ConvertPathFromUtf8(mn_dir));
-    fs::create_directory(mn_dir);
-    WorkCnt++;
+    PixCanvasBuf&  cnv_buf = GetTaggedPCB(mn, data_tag);
+    RefPtr<Gdk::Pixbuf> cnv_pix = cnv_buf.Source();
+    // из-за того, что при lb-режиме не будет рисовать на "матах"
+    cnv_pix->fill(TRANS_CLR);
 
     RectListRgn rct_lst;
     rct_lst.push_back(cnv_buf.FramePlacement());
+    
+    const char* highlight_fname = not_letterbox16_9 ? "MenuHighlight.png" : "MenuHighlightLB.png" ;
+    const char* select_fname    = not_letterbox16_9 ? "MenuSelect.png"    : "MenuSelectLB.png" ;
+    const char* spumux_fname    = not_letterbox16_9 ? "Menu.xml" : "MenuLB.xml" ;
+    
     xmlpp::Document doc;
     xmlpp::Element* root_node = doc.create_root_node("subpictures");
     xmlpp::Element* spu_node = root_node->add_child("stream")->add_child("spu");
     spu_node->set_attribute("start", "00:00:00.0");
     spu_node->set_attribute("end",   "00:00:00.0");
-    spu_node->set_attribute("highlight", "MenuHighlight.png");
-    spu_node->set_attribute("select",    "MenuSelect.png");
+    spu_node->set_attribute("highlight", highlight_fname);
+    spu_node->set_attribute("select",    select_fname);
     // используем "родной" прозрачный цвет в png
     //std::string trans_color = ColorToString(BLACK2_CLR);
     //spu_node->set_attribute("transparent", trans_color);
@@ -327,25 +386,40 @@ bool RenderSubPictures(const std::string& out_dir, Menu mn, int i,
     spu_node->set_attribute("autoorder",   "rows");
 
     // * активация
-    SPRenderVis r_vis(rct_lst, spu_node);
+    SPRenderVis r_vis(rct_lst, spu_node, not_letterbox16_9);
     m_rgn.Accept(r_vis);
-    SaveMenuPicture(mn_dir, "MenuHighlight.png", cnv_pix);
-    doc.write_to_file_formatted(AppendPath(mn_dir, "Menu.xml"));
+    SaveMenuPicture(mn_dir, highlight_fname, cnv_pix);
+    doc.write_to_file_formatted(AppendPath(mn_dir, spumux_fname));
 
     // * выбор
     r_vis.SetSelect(true);
     m_rgn.Accept(r_vis);
-    SaveMenuPicture(mn_dir, "MenuSelect.png", cnv_pix);
+    SaveMenuPicture(mn_dir, select_fname, cnv_pix);
+}
 
-    // * остальное
-    //fs::copy_file(SConsAuxDir()/"menu_SConscript", fs::path(mn_dir)/"SConscript");
-    std::string str = ReadAllStream(SConsAuxDir()/"menu_SConscript");
-    bool is_motion  = mn->MtnData().isMotion;
-    str = boost::format(str) % (is_motion ? "True" : "False") % bf::stop;
-    WriteAllStream(fs::path(mn_dir)/"SConscript", str);
+bool RenderSubPictures(const std::string& out_dir, Menu mn, int i, 
+                       std::iostream& menu_list)
+{
+    // :WARN: правильно ли обращаемся (на питоне) к файловой системе не в utf8?
+    std::string mn_dir = MenuAuthorDir(mn, i, false);
+    menu_list << "'''" << mn_dir << "''',\n"; 
+    mn_dir = AppendPath(out_dir, ConvertPathFromUtf8(mn_dir));
+    fs::create_directory(mn_dir);
+    WorkCnt++;
+
+    MakeSubpictures(mn, true, mn_dir);
+    if( !IsMenuToBe4_3() )
+        MakeSubpictures(mn, false, mn_dir);
+    
+    //// остальное
+    ////fs::copy_file(SConsAuxDir()/"menu_SConscript", fs::path(mn_dir)/"SConscript");
+    //std::string str = ReadAllStream(SConsAuxDir()/"menu_SConscript");
+    //bool is_motion  = mn->MtnData().isMotion;
+    //str = boost::format(str) % (is_motion ? "True" : "False") % bf::stop;
+    //WriteAllStream(fs::path(mn_dir)/"SConscript", str);
 
     // для последующего рендеринга
-    SetCBDirty(cnv_buf);
+    SetCBDirty(GetAuthorPCB(mn));
 
     return true;
 }
@@ -423,9 +497,8 @@ RefPtr<Gdk::Pixbuf> FTOAuthorData::CalcSource(Project::MediaItem mi, const Point
 
 void CommonMenuRVis::RenderStatic(FrameThemeObj& fto)
 {
-    // используем кэш
     FTOAuthorData& pix_data = fto.GetData<FTOAuthorData>(AUTHOR_TAG);
-    drw->CompositePixbuf(pix_data.GetPix(), CalcRelPlacement(fto.Placement()));
+    DoRenderFTO(drw.get(), fto, pix_data, cnvBuf->Transition());
 }
 
 void MenuRenderVis::Visit(FrameThemeObj& fto)
@@ -442,20 +515,45 @@ static bool IsToBeMoving(MediaItem mi)
     return mi && (IsVideo(mi) || IsChapter(mi));
 }
 
-static Rect RealPosition(Comp::MediaObj& obj, const Planed::Transition& trans)
+const char* AVCnvBin()
 {
-    return AbsToRel(trans, obj.Placement());
+    // :TRICKY: под Win32 нельзя запускать (пока) несущ. бинари без вылета;
+    // с другой стороны, runtime-выбор конвертора (!) - тоже вынужденность из-за
+    // множества конфигураций в Linux'е 
+#ifdef _WIN32
+    return "ffmpeg";
+#else
+    static std::string cnv_name;
+    if( !cnv_name.size() )
+    {
+        // Поведение execv() довольно сложно, если путь не абслютный, см. спеки и eglibc/posix/execvp.c
+        // (все на стороне пользователя):
+        // - узнается список путей для поиска:
+        //   - проверяется переменная PATH
+        //   - если пусто, то берется confstr(_CS_PATH), которое зависит от платформы 
+        //     (можно посмотреть с "getconf CS_PATH", под Linux - "/bin:/usr/bin") и прибавляется в конец
+        //     ":."
+        // - в цикле проверяются все абсолютные пути явным запуском ядерного вызова execve
+        // - если ошибка ENOEXEC (не выполняемый формат файла; например, в случае скриптов), то запускается  
+        //   "/bin/sh" с этим скриптом
+        // 
+        // Вывод: идем простым путем
+        std::string output;
+        cnv_name = PipeOutput("avconv -version", output).IsGood() ? "avconv" : "ffmpeg";
+    }
+    return cnv_name.c_str();
+#endif
 }
 
 static void FFmpegError(const ExitData& ed)
 {
     //Author::ErrorByED(FFmpegErrorTemplate(), ed);
-    Author::ApplicationError("ffmpeg", ed);
+    Author::ApplicationError(AVCnvBin(), ed);
 }
 
 static void FFmpegError(const std::string& msg)
 {
-    Author::ApplicationError("ffmpeg", msg);
+    Author::ApplicationError(AVCnvBin(), msg);
 }
 
 static std::string errno2str()
@@ -515,14 +613,11 @@ static Point DVDAspect(bool is4_3)
     return is4_3 ? Point(4, 3) : Point(16,9);
 }
 
-AutoDVDTransData::AutoDVDTransData(bool is4_3_): is4_3(is4_3_), 
-    srcAspect(DVDAspect(is4_3_)), audioNum(1) {}
+AutoSrcData::AutoSrcData(bool is4_3_): videoOK(false),
+    dar(DVDAspect(is4_3_)), audioNum(1) {}
 
-template<typename DstT, typename SrcT>
-static PointT<DstT> ChangePoint(const PointT<SrcT>& pnt)
-{
-    return PointT<DstT>(pnt.x, pnt.y);
-}
+AutoDVDTransData::AutoDVDTransData(bool is4_3_): is4_3(is4_3_), 
+    asd(is4_3_), threadsCnt(1) {}
 
 static int PadSize(int free_space)
 {
@@ -579,14 +674,20 @@ static std::string FFSizeSet(const Point& sz, int val,
     return sz_str;
 }
 
+static void AppendOpts(std::string& opts, const std::string& append_args)
+{
+    if( !append_args.empty() )
+        opts += " " + append_args;
+}
+
 std::string FFmpegToDVDArgs(const std::string& out_fname, const AutoDVDTransData& atd, bool is_pal,
                             const DVDTransData& td)
 {
     // * предполагается после создания команды ее вызов => требуется
     // проверка такой возможности
-    TripleVersion filter_ver = CheckFFDVDEncoding().avfilter;
+    FFmpegVersion ff_ver = CheckFFDVDEncoding();
     // вместо -padX использовать -vf pad=w:h:x:y:black
-    bool is_vf = IsVersionGE(filter_ver, TripleVersion(1, 20, 0));
+    bool is_vf = IsVersionGE(ff_ver.avfilter, TripleVersion(1, 20, 0));
 
     // *
     const char* target =  is_pal ? "pal" : "ntsc" ;
@@ -599,15 +700,13 @@ std::string FFmpegToDVDArgs(const std::string& out_fname, const AutoDVDTransData
     ASSERT( vrate >= 0 );
     if( vrate )
         bitrate_str = boost::format("-b %1%k ") % vrate % bf::stop;
-    bitrate_str += boost::format("-ab %1%k ") % TRANS_AUDIO_BITRATE % bf::stop;
+    bitrate_str += boost::format("%2% %1%k") % TRANS_AUDIO_BITRATE 
+        % (IsVersionGE(ff_ver.avcodec, TripleVersion(53, 25, 0)) ? "-b:a" : "-ab") % bf::stop;
 
     // * соотношение
     bool is_4_3 = atd.is4_3;
     // * размеры
-    DPoint dst_asp = ChangePoint<double>(DVDAspect(is_4_3));
-    DPoint asp = FitInto1(dst_asp, ChangePoint<double>(atd.srcAspect));
-
-    Point img_sz(asp.x * sz.x, asp.y * sz.y);
+    Point img_sz = FitIntoRect(sz, DVDAspect(is_4_3), atd.asd.dar).Size();
     std::string sz_str = FFSizeOption(sz);
     if( img_sz.x < sz.x )
     {
@@ -623,21 +722,36 @@ std::string FFmpegToDVDArgs(const std::string& out_fname, const AutoDVDTransData
         if( val )
             sz_str = FFSizeSet(sz, val, false, is_vf);
     }
-    // * дополнительные аудио
-    std::string add_audio_str;
-    for( int i=0; i<(atd.audioNum-1); i++ )
-        // ffmpeg обнуляет audio_codec_name после каждого -newaudio и -i
-        add_audio_str += " -acodec ac3 -newaudio";
 
     // * доп. опции
     std::string add_opts("-mbd rd -trellis 2 -cmp 2 -subcmp 2");
-    {
-        std::string user_opts = PrefContents("ffmpeg_options");
-        if( !user_opts.empty() )
-            add_opts += " " + user_opts;
-    }
+    if( atd.asd.videoOK )
+        AppendOpts(add_opts, "-vcodec copy");
+    if( atd.threadsCnt > 1 )
+        AppendOpts(add_opts, boost::format("-threads %1%") % atd.threadsCnt % bf::stop);
+    AppendOpts(add_opts, PrefContents("ffmpeg_options"));
+    AppendOpts(add_opts, td.ctmFFOpt);
 
-    return boost::format("-target %1%-dvd -aspect %2% %3% %4%-y %7% %5%%6%")
+    // * дополнительные аудио
+    std::string add_audio_str;
+    int anum = atd.asd.audioNum;
+    if( IsVersionGE(ff_ver.avcodec, TripleVersion(53, 15, 0)) )
+    {
+        // :KLUDGE: 1) не учитываем audioNum, но пока все равно пользователь
+        // не может его изменить
+        // 2) видео явно записывать приходится из-за использования -map => берем одно=первое
+        // (алгоритм без -map выбирает с наибольшим разрешением)
+        if( anum > 1 )
+            AppendOpts(add_opts, "-map 0:v:0 -map 0:a");
+    }
+    else
+    {
+        for( int i=0; i<(anum-1); i++ )
+            // ffmpeg обнуляет audio_codec_name после каждого -newaudio и -i
+            add_audio_str += " -acodec ac3 -newaudio";
+    }
+    
+    return boost::format("-target %1%-dvd -aspect %2% %3% %4% -y %7% %5%%6%")
         % target % (is_4_3 ? "4:3" : "16:9") % sz_str
         % bitrate_str % FilenameForCmd(out_fname) % add_audio_str % add_opts % bf::stop;
 }
@@ -650,13 +764,20 @@ std::string FFmpegToDVDArgs(const std::string& out_fname, bool is_4_3, bool is_p
     return FFmpegToDVDArgs(out_fname, is_4_3, is_pal, td);
 }
 
+static ExitData AVCnvOutput(const std::string& args, std::string& output)
+{
+    return PipeOutput(std::string(AVCnvBin()) + " " + args, output);
+}
+
 std::string FFmpegPostArgs(const std::string& out_fname, bool is_4_3, bool is_pal, 
-                           const std::string& a_fname, double a_shift)
+                           const AudioArgInput& aai) //const std::string& a_fname, double a_shift)
 {
     std::string a_input = "-an"; // без аудио
-    if( !a_fname.empty() )
+
+    std::string a_fname = aai.fName;
+    if( a_fname.size() )
     {
-        std::string safe_a_fname = FilenameForCmd(a_fname);
+        a_fname = FilenameForCmd(a_fname);
         int idx = -1;
         // в последних версиях ffmpeg (черт побери) поменяли отображение
         // входных потоков на выходные, в итоге возникают проблемы с указанием
@@ -669,25 +790,33 @@ std::string FFmpegPostArgs(const std::string& out_fname, bool is_4_3, bool is_pa
             // считается "ошибочным" (но стандарт де факто для получения инфо
             // о файле через ffmpeg)
             std::string a_info;
-            PipeOutput("ffmpeg -i " + safe_a_fname, a_info);
+            AVCnvOutput("-i " + a_fname, a_info);
 
-            static re::pattern audio_idx("Stream #"RG_NUM"\\."RG_NUM".*Audio:");
+            // нужно второе число из информации по первому аудиопотоку; оно равно
+            // индексу этого потока в массиве AVFormatContext.streams (их число -
+            // AVFormatContext.nb_streams; , см. проверку в opt_map:
+            // check_stream_specifier(input_files[file_idx].ctx, input_files[file_idx].ctx->streams[i])
+            // 
+            // :KLUDGE: (только) в ffmpeg, avformat 53.13.0, поменяли . на : => надо
+            // самим открывать файл и узнавать индекс! 
+            static re::pattern audio_idx("Stream #"RG_NUM"[\\.|:]"RG_NUM".*Audio:");
 
             re::match_results what;
             // флаг означает, что перевод строки не может быть точкой
             if( re::search(a_info, what, audio_idx, boost::match_not_dot_newline) )
                 idx = boost::lexical_cast<int>(what.str(2));
             else
-                FFmpegError(BF_("no audio stream in %1%") % safe_a_fname % bf::stop);
+                FFmpegError(BF_("no audio stream in %1%") % a_fname % bf::stop);
         }
         ASSERT( idx >= 0 );
-        // каждая опция -map задает соответ. одному выходному потоку входной поток
+        // каждая опция -map добавляет (до libav 0.8 - только назначает) один поток из файла 
+        // под номером до ":" и с номерем потока в том файле - после ":" 
         std::string map = boost::format("-map 0:0 -map 1:%1%") % idx % bf::stop;
 
         std::string shift; // без смещения
-        if( a_shift )
-            shift = boost::format("-ss %.2f ") % a_shift % bf::stop;
-        a_input = boost::format("%2%-i %1% %3%") % safe_a_fname % shift % map % bf::stop; 
+        if( aai.shift )
+            shift = boost::format("-ss %.2f ") % aai.shift % bf::stop;
+        a_input = boost::format("%2%-i %1% %3%") % a_fname % shift % map % bf::stop; 
     }
 
     return a_input + " " + FFmpegToDVDArgs(out_fname, is_4_3, is_pal);
@@ -739,8 +868,7 @@ static std::string GetFilename(VideoStart vs)
     return GetFilename(*vs.first);
 }
 
-static void LoadMotionFrame(RefPtr<Gdk::Pixbuf>& pix, MediaItem ref, DataWare& src,
-                            MotionTimer& mt)
+static RefPtr<Gdk::Pixbuf> GetMotionFrame(MediaItem ref, DataWare& src, MotionTimer& mt)
 {
     VideoStart vs = GetVideoStart(ref);
     VideoViewer& plyr = src.GetData<VideoViewer>(MOTION_MENU_TAG);
@@ -748,7 +876,13 @@ static void LoadMotionFrame(RefPtr<Gdk::Pixbuf>& pix, MediaItem ref, DataWare& s
         RGBOpen(plyr, GetFilename(vs));
 
     double tm = vs.second + mt.Time();
-    GetFrame(pix, tm, plyr);
+    return VideoPE(plyr, tm).Make(Point()).ROPixbuf();
+}
+
+static void LoadMotionFrame(RefPtr<Gdk::Pixbuf>& pix, MediaItem ref, DataWare& src,
+                            MotionTimer& mt)
+{
+    RGBA::CopyOrScale(pix, GetMotionFrame(ref, src, mt));
 }
 
 void MotionMenuRVis::RenderBackground()
@@ -756,16 +890,17 @@ void MotionMenuRVis::RenderBackground()
     Menu mn = GetOwnerMenu(menuRgn);
     MotionTimer& mt  = GetMotionTimer(mn);
     MediaItem bg_ref = menuRgn->BgRef();
-    RefPtr<Gdk::Pixbuf> menu_pix = cnvBuf->FramePixbuf();
+    Rect plc = cnvBuf->FrameRect();
 
     if( IsToBeMoving(bg_ref) )
     {
-        // явно отображаем кадр в стиле монитора (DAMonitor),
-        // а не редактора, потому что:
-        // - область и так всю перерисовывать
-        // - самый быстрый вариант (без каких-либо посредников)
-        //drw->ScalePixbuf(pix, cnvBuf->FrameRect());
-        LoadMotionFrame(menu_pix, bg_ref, *mn, mt);
+        //// явно отображаем кадр в стиле монитора (DAMonitor),
+        //// а не редактора, потому что:
+        //// - область и так всю перерисовывать
+        //// - самый быстрый вариант (без каких-либо посредников)
+        ////drw->ScalePixbuf(pix, cnvBuf->FrameRect());
+        //LoadMotionFrame(menu_pix, bg_ref, *mn, mt);
+        DoRenderBackground(drw, GetMotionFrame(bg_ref, *mn, mt), menuRgn, plc);
     }
     else
     {
@@ -776,21 +911,21 @@ void MotionMenuRVis::RenderBackground()
             // потому что пока вообще пусто (а область рендеринга сужена
             // до анимационных элементов)
             //MyParent::RenderBackground();
-            RGBA::Scale(menu_pix, static_menu_pix);
+            RGBA::Scale(cnvBuf->FramePixbuf(), static_menu_pix);
         }
         else
-            drw->ScalePixbuf(static_menu_pix, cnvBuf->FrameRect());
+            drw->ScalePixbuf(static_menu_pix, plc);
     }
 }
 
 void MotionMenuRVis::Visit(FrameThemeObj& fto)
 {
     MediaItem mi = MIToDraw(fto);
-    if( IsToBeMoving(mi) )
+    if( IsToBeMoving(mi) && !IsIconTheme(fto) )
     {
         // напрямую делаем то же, что FTOData::CompositeFTO(),
         // чтобы явно видеть затраты
-        const Editor::ThemeData& td = Editor::ThemeCache::GetTheme(fto.Theme());
+        const Editor::ThemeData& td = Editor::ThemeCache::GetTheme(fto.ThemeName());
         // нужен оригинал в размере vFrameImg
         RefPtr<Gdk::Pixbuf> obj_pix = td.vFrameImg->copy();
         LoadMotionFrame(obj_pix, mi, fto, GetMotionTimer(GetOwnerMenu(&fto)));
@@ -808,35 +943,38 @@ bool IsMotion(Menu mn)
     return mn->MtnData().isMotion;
 }
 
-bool IsMenuToBe4_3();
-
 static void SaveMenuPng(const std::string& mn_dir, Menu mn)
 {
     SaveMenuPicture(mn_dir, "Menu.png", GetAuthoredMenu(mn));
 }
 
-static std::string MakeFFmpegPostArgs(const std::string& mn_dir, Menu mn)
+static AudioArgInput MotionMenuAAI(Menu mn)
 {
+    ASSERT( IsMotion(mn) );
     MotionData& mtn_dat = mn->MtnData();
-    // аспект ставим как можем (а не из параметров меню) из-за того, что качество
-    // авторинга важнее гибкости (все валим в один titleset)
-    bool is_4_3 = IsMenuToBe4_3();
-    std::string out_fname = AppendPath(mn_dir, "Menu.mpg");
-
-    std::string a_fname;
-    double a_shift = 0.;
+    AudioArgInput aai;
     if( mtn_dat.isIntAudio )
     {
         if( MediaItem a_ref = mtn_dat.audioRef.lock() )
         {
             VideoStart vs = GetVideoStart(a_ref);
-            a_fname = GetFilename(vs);
-            a_shift = vs.second;
+            aai.fName = GetFilename(vs);
+            aai.shift = vs.second;
         }
     }
     else
-        a_fname = mtn_dat.audioExtPath;
-    return FFmpegPostArgs(out_fname, is_4_3, IsPALProject(), a_fname, a_shift);
+        aai.fName = mtn_dat.audioExtPath;
+    return aai;
+}
+
+static std::string MakeFFmpegPostArgs(const std::string& mn_dir, const AudioArgInput& aai)
+{
+    // аспект ставим как можем (а не из параметров меню) из-за того, что качество
+    // авторинга важнее гибкости (все валим в один titleset)
+    bool is_4_3 = IsMenuToBe4_3();
+    std::string out_fname = AppendPath(mn_dir, "Menu.mpg");
+
+    return FFmpegPostArgs(out_fname, is_4_3, IsPALProject(), aai);
 }
 
 double MenuDuration(Menu mn)
@@ -848,7 +986,7 @@ double MenuDuration(Menu mn)
     return duration;
 }
 
-static Gtk::TextView& PrintCmdToDetails(const std::string& cmd)
+Gtk::TextView& PrintCmdToDetails(const std::string& cmd)
 {
     Gtk::TextView& tv = Author::GetES().detailsView;
     AppendCommandText(tv, cmd);
@@ -856,38 +994,51 @@ static Gtk::TextView& PrintCmdToDetails(const std::string& cmd)
 }
 
 void RunExtCmd(const std::string& cmd, const char* app_name, 
-               const ReadReadyFnr& add_fnr)
+               const ReadReadyFnr& add_fnr, const char* dir)
 {
     ExitData ed;
     if( Execution::ConsoleMode::Flag )
     {
         io::cout << cmd << io::endl;
-        ed = System(cmd);
+        io::cout << "In directory: " << (dir ? dir : "<of the process>") << io::endl;
+        if( dir )
+            ed = WaitForExit(Spawn(dir, cmd.c_str(), 0, true));
+        else
+            ed = System(cmd);
     }
     else
-    {
-        Gtk::TextView& tv = PrintCmdToDetails(cmd);
-        ed = Author::AsyncCall(0, cmd.c_str(), TextViewAppender(tv, add_fnr));
-    }
-    if( !ed.IsGood() )
-        Author::ApplicationError(app_name, ed);
+        ed = Author::AsyncCall(dir, cmd.c_str(), DetailsAppender(cmd, add_fnr));
+    Author::CheckAppED(ed, app_name);
 }
 
 void RunFFmpegCmd(const std::string& cmd, const ReadReadyFnr& add_fnr)
 {
-    RunExtCmd(cmd, "ffmpeg", add_fnr);
+    RunExtCmd(cmd, AVCnvBin(), add_fnr);
+}
+
+static FFmpegVersion CalcFFmpegVersion();
+
+// кодируем из Menu.png => Menu.mpg
+static void EncodeStillMenu(const std::string& mn_dir, double durtn, const AudioArgInput& aai)
+{
+    std::string img_fname = AppendPath(mn_dir, "Menu.png");
+
+    // :TRICKY: в libav на замену -loop_input ввели -loop, в libavformat 53.2.0; в ffmpeg оно попало,
+    // когда версия в libavformat была уже 53.6.0 или около того, и merge понизил версию опять до 53.2.0
+    // (да, вот такой бред теперь происходит из-за войны ffmpeg vs libav => см. в git'е ffmpeg так:
+    // git log -p  -- libavformat/version.h | grep LIBAVFORMAT_VERSION_MINOR | less
+    const char* loop_opt = IsVersionGE(CalcFFmpegVersion().avformat, TripleVersion(53, 6, 0)) ? "-loop 1" : "-loop_input" ;
+    //std::string ffmpeg_cmd = boost::format("%4% -t %3$.2f -loop_input -i \"%1%\" %2%")
+    std::string ffmpeg_cmd = boost::format("%4% %5% -i \"%1%\" -t %3$.2f %2%") 
+        % img_fname % MakeFFmpegPostArgs(mn_dir, aai) % durtn % AVCnvBin() % loop_opt % bf::stop;
+
+    RunFFmpegCmd(ffmpeg_cmd);
 }
 
 static void SaveStillMenuMpg(const std::string& mn_dir, Menu mn)
 {
-    // сохраняем Menu.png и рендерим из нее
     SaveMenuPng(mn_dir, mn);
-    std::string img_fname = AppendPath(mn_dir, "Menu.png");
-
-    std::string ffmpeg_cmd = boost::format("ffmpeg -t %3$.2f -loop_input -i \"%1%\" %2%") 
-        % img_fname % MakeFFmpegPostArgs(mn_dir, mn) % MenuDuration(mn) % bf::stop;
-
-    RunFFmpegCmd(ffmpeg_cmd);
+    EncodeStillMenu(mn_dir, MenuDuration(mn), MotionMenuAAI(mn));
 }
 
 FFmpegCloser::~FFmpegCloser()
@@ -995,15 +1146,8 @@ TripleVersion FindVersion(const std::string& conts, const re::pattern& ver_pat,
 }
 
 // conts - вывод ffmpeg -formats
-FFmpegVersion TestFFmpegForDVDEncoding(const std::string& conts)
+void TestFFmpegForDVDEncoding(const std::string& conts)
 {
-    // ищем версию libavfilter
-    // пример: " libavfilter    0. 4. 0 / "
-#define RG_PADNUM RG_SPS RG_NUM
-    static re::pattern avfilter_version(RG_BW"libavfilter"RG_PADNUM"\\."RG_PADNUM"\\."RG_PADNUM" / ");
-    FFmpegVersion ff_ver;
-    ff_ver.avfilter = FindVersion(conts, avfilter_version, "ffmpeg", "libavfilter");
-
     CheckNoCodecs(CheckForCodecList(conts));
 
     static re::pattern dvd_format("^ .E dvd"RG_EW);
@@ -1015,41 +1159,74 @@ FFmpegVersion TestFFmpegForDVDEncoding(const std::string& conts)
     // по факту ffmpeg всегда использует ac3, однако mp2 тоже возможен
     static re::pattern ac3_codec("^ .EA... ac3"RG_EW);
     CheckStrippedFFmpeg(ac3_codec, conts, "ac3 audio encoder");
-
-    return ff_ver;
 }
 
-FFmpegVersion CheckFFDVDEncoding()
+TripleVersion FindAVVersion(const std::string& conts, const char* avlib_name)
 {
-    FFmpegVersion ff_ver;
-    // наличие полного ffmpeg
-    std::string ff_formats;
-    ExitData ed = PipeOutput("ffmpeg -formats", ff_formats);
-    // старые версии выходят с 1 для нерабочих режимов -formats, -help, ... (Hardy)
-    bool is_good = ed.IsGood() || ed.IsCode(1);
-    if( !is_good )
+    // * ищем версию libavfilter
+    // пример: " libavfilter    0. 4. 0 / "
+#define RG_PADNUM RG_SPS RG_NUM
+    std::string reg_str = boost::format(RG_BW"%1%"RG_PADNUM"\\."RG_PADNUM"\\."RG_PADNUM" / ")
+        % avlib_name % bf::stop;
+    re::pattern avfilter_version(reg_str.c_str());
+    return FindVersion(conts, avfilter_version, AVCnvBin(), avlib_name);
+}
+
+TripleVersion FindAVFilterVersion(const std::string& conts)
+{
+    return FindAVVersion(conts, "libavfilter");
+}
+
+static FFmpegVersion CalcFFmpegVersion()
+{
+    // *
+    std::string conts;
+    ExitData ed = AVCnvOutput("-version", conts);
+    if( !ed.IsGood() )
     {
         const char* msg = ed.IsCode(127) ? _("command not found") : "unknown error" ;
         FFmpegError(msg);
     }
-    else
+    
+    // *
+    FFmpegVersion ff_ver;
+    ff_ver.avcodec  = FindAVVersion(conts, "libavcodec");
+    ff_ver.avformat = FindAVVersion(conts, "libavformat");
+    ff_ver.avfilter = FindAVFilterVersion(conts);
+    return ff_ver;
+}
+
+// :TRICKY: сейчас делается 3 вызова бинаря; можно свести к двум,
+// если использовать -v verbose (без -version, необходимость которого
+// возникла из-за avconv), но это работает 
+// только для >= ffmpeg 0.8.5
+FFmpegVersion CheckFFDVDEncoding()
+{
+    FFmpegVersion ff_ver = CalcFFmpegVersion();
+
+    // * наличие полного ffmpeg
+    std::string ff_formats;
+    ExitData ed = AVCnvOutput("-formats", ff_formats);
+    // старые версии выходят с 1 для нерабочих режимов -formats, -help, ... (Hardy)
+    bool is_good = ed.IsGood() || ed.IsCode(1);
+    if( !is_good )
+        FFmpegError("-formats");
+    
+    // * в новых версиях ffmpeg распечатка собственно кодеков выделена
+    //  в опцию -codecs
+    // * нормальной (сквозной) нумерации собственно ffmpeg не имеет (см. version.sh,-
+    // может появиться UNKNOWN, SVN-/git-ревизия и собственно версия), потому
+    // определяем по наличию строки "Codecs:"
+    if( !CheckForCodecList(ff_formats) )
     {
-        // * в новых версиях ffmpeg распечатка собственно кодеков выделена
-        //  в опцию -codecs
-        // * нормальной (сквозной) нумерации собственно ffmpeg не имеет (см. version.sh,-
-        // может появиться UNKNOWN, SVN-/git-ревизия и собственно версия), потому
-        // определяем по наличию строки "Codecs:"
-        if( !CheckForCodecList(ff_formats) )
-        {
-            std::string ff_codecs;
-            ExitData ed = PipeOutput("ffmpeg -codecs", ff_codecs);
-            CheckNoCodecs(ed.IsGood());
+        std::string ff_codecs;
+        ExitData ed = AVCnvOutput("-codecs", ff_codecs);
+        CheckNoCodecs(ed.IsGood());
 
-            ff_formats += ff_codecs;
-        }
-
-        ff_ver = TestFFmpegForDVDEncoding(ff_formats);
+        ff_formats += ff_codecs;
     }
+
+    TestFFmpegForDVDEncoding(ff_formats);
     return ff_ver;
 }
 
@@ -1077,10 +1254,10 @@ bool RenderMainPicture(const std::string& out_dir, Menu mn, int i)
                 rlr.push_back(mtn_buf.FrameRect());
             else
             {
-                boost_foreach( Comp::Object* obj, m_rgn.List() )
+                boost_foreach( Comp::MediaObj* obj, m_rgn.List() )
                     if( FrameThemeObj* frame_obj = dynamic_cast<FrameThemeObj*>(obj) )
                         if( IsToBeMoving(MIToDraw(*frame_obj)) )
-                            rlr.push_back(RealPosition(*frame_obj, mtn_buf.Transition()));
+                            AddRelPos(rlr, frame_obj, mtn_buf.Transition());
             }
             ReDivideRects(rlr);
 
@@ -1094,8 +1271,8 @@ bool RenderMainPicture(const std::string& out_dir, Menu mn, int i)
                 // 1. Ставим частоту fps впереди -i pipe, чтобы она воспринималась для входного потока
                 // (а для выходного сработает -target)
                 // 2. И наоборот, для выходные параметры (-aspect, ...) ставим после всех входных файлов
-                std::string ffmpeg_cmd = boost::format("ffmpeg -r %1% -f image2pipe -vcodec ppm -i pipe: %2%")
-                    % MotionTimer::fps % MakeFFmpegPostArgs(mn_dir, mn) % bf::stop;
+                std::string ffmpeg_cmd = boost::format("%3% -r %1% -f image2pipe -vcodec ppm -i pipe: %2%")
+                    % MotionTimer::fps % MakeFFmpegPostArgs(mn_dir, MotionMenuAAI(mn)) % AVCnvBin() % bf::stop;
 
                 ExitData ed;
                 {
@@ -1115,11 +1292,9 @@ bool RenderMainPicture(const std::string& out_dir, Menu mn, int i)
                     }
                     else
                     {
-                        Gtk::TextView& tv = PrintCmdToDetails(ffmpeg_cmd);
                         // закрываем выходы после окончания работы ffmpeg, иначе тот грохнется
                         // по SIGPIPE
-                        //OutErrBlock oeb(out_err, TextViewAppender(tv));
-                        pc.oeb = new OutErrBlock(out_err, TextViewAppender(tv));
+                        pc.oeb = new OutErrBlock(out_err, DetailsAppender(ffmpeg_cmd));
 
                         PipeVideo(mn, pc.inFd);
                     }
@@ -1136,7 +1311,21 @@ bool RenderMainPicture(const std::string& out_dir, Menu mn, int i)
         }
     }
     else
+    {
         SaveMenuPng(mn_dir, mn);
+        // :TRICKY: значение 0.1 секунда - эмпирическое (в скрипте ADVD.py ставить
+        // равное значение); по опыту c mpeg2enc кодировать нужно > одного кадра,
+        // чтобы было качество; вероятно подобное применимо и к ffmpeg
+        EncodeStillMenu(mn_dir, 0.1, AudioArgInput());
+    }
+
+    if( IsMenuToBe4_3() )
+        RunSpumux("Menu.xml", "Menu.mpg", "MenuSub.mpg", 0, mn_dir.c_str());
+    else
+    {
+        RunSpumux("Menu.xml", "Menu.mpg", "MenuSub0.mpg", 0, mn_dir.c_str());
+        RunSpumux("MenuLB.xml", "MenuSub0.mpg", "MenuSub.mpg", 1, mn_dir.c_str());
+    }
 
     PulseRenderProgress();
     return true;
