@@ -33,6 +33,7 @@
 #include <mgui/project/thumbnail.h> // Project::CalcAspectSize()
 #include <mgui/project/video.h>
 #include <mgui/project/handler.h>
+#include <mgui/prefs.h>
 #include <mgui/gettext.h>
 
 #include <mbase/project/table.h>
@@ -58,6 +59,13 @@ static xmlpp::Element* AddVideoTag(xmlpp::Element* node, bool is_4_3 = true)
     xmlpp::Element* vnode = node->add_child("video");
     AddFormatAttr(vnode);
     vnode->set_attribute("aspect", is_4_3 ? "4:3" : "16:9");
+    if( !is_4_3 )
+        // в режиме 16:9 ресурсы (36) тратятся на 3 режима: основной (widescreen),
+        // letterbox и pan&scan; последний дефакто не используется + кол-во
+        // кнопок для меню увеличиваем до 16; см. также
+        // http://forum.videohelp.com/threads/298871-More-than-12-buttons-with-dvdauthor
+        // :TRICKY: нужно для меню, для видео факт неочевиден
+        vnode->set_attribute("widescreen", "nopanscan");
 
     return vnode;
 }
@@ -71,7 +79,7 @@ TSBegEnd BeginEnd(RefPtr<Gtk::TreeStore> store)
 
 // из-за того, что все медиа в одном браузере (включая картинки="не видео"),
 // нельзя использовать естественную нумерацию для авторинга 
-static int& GetAuthorNumber(VideoItem vi)
+int& GetAuthorNumber(VideoItem vi)
 {
     //return GetBrowserPath(&obj)[0]+1;
     return vi->GetData<int>(AUTHOR_TAG);
@@ -79,7 +87,7 @@ static int& GetAuthorNumber(VideoItem vi)
 
 int ForeachVideo(VideoFnr fnr)
 {
-    RefPtr<MediaStore> md_store = GetAStores().mdStore;
+    RefPtr<MediaStore> md_store = GetMediaStore();
     TSBegEnd medias_be          = BeginEnd(md_store);
     int i = 0;
     for( Gtk::TreeIter itr = medias_be.first; itr != medias_be.second; ++itr )
@@ -163,11 +171,15 @@ void TargetCommandVis::Visit(VideoMD& obj)
 
 void TargetCommandVis::Visit(VideoChapterMD& obj) 
 {
-    int v_num = GetAuthorNumber(obj.owner);
-    // :TODO: title 1 всегда равно title 1 chapter 1; при этом dvdauthor не воспринимает 
-    // главы "вблизи нуля" (<0.27секунд; задание - посмотреть точно), поэтому нужна предварительная
-    // перенумерация, как с видео
-    res = (str::stream() << "jump title " << v_num << " chapter " << ChapterPosInt(&obj) + 2 << ";").str();
+    VideoMD* owner = obj.owner;
+    int v_num = GetAuthorNumber(owner);
+    // title 1 всегда равно title 1 chapter 1 (первая ячейка всегда создавется), и максим. кол-во адресуемых 
+    // номеров <= кол-во bmd-глав+1; если 2 главы слишком близки к друг другу (внутри одного vobu=область между
+    // двумя NAV-пакетами, ~0.5 секунды), то dvdauthor их "склеивает" в одну => кол-во номеров уменьшается
+    // Потому: для удоства пользователей даем создавать нулевую главу, разрешая это здесь 
+    // (однако доп. нулевые главы будут приводить к ошибке Cannot jump to chapter N ... only M exist)
+    int c_num = ChapterPosInt(&obj) + (owner->List()[0]->chpTime ? 2 : 1) ;
+    res = (str::stream() << "jump title " << v_num << " chapter " << c_num << ";").str();
 }
 
 static std::string MakeButtonJump(MediaItem mi, bool vts_domain)
@@ -224,6 +236,12 @@ bool HasButtonLink(Comp::MediaObj& m_obj, std::string& targ_str)
     return res;
 }
 
+bool HasButtonLink(Comp::MediaObj& m_obj)
+{
+    std::string targ_str;
+    return Project::HasButtonLink(m_obj, targ_str);
+}
+
 int MenusCnt()
 {
     return Size(GetAStores().mnStore);
@@ -335,14 +353,12 @@ static bool ScriptMenu(xmlpp::Element* menus_node, Menu root_menu, Menu mn, int
         AddPostCmd(pgc_node, mn);
 
     // действия кнопок
-    MenuRegion::ArrType& lst = GetMenuRegion(mn).List();
-    for( MenuRegion::Itr itr = lst.begin(), end = lst.end(); itr != end; ++itr )
-        if( Comp::MediaObj* m_obj = dynamic_cast<Comp::MediaObj*>(*itr) )
-        {
-            std::string targ_str;    
-            if( HasButtonLink(*m_obj, targ_str) )
-                AddChildWithText(pgc_node, "button", targ_str);
-        }
+    boost_foreach( Comp::MediaObj* m_obj, Author::AllMediaObjs(mn) )
+    {
+	std::string targ_str;    
+	if( HasButtonLink(*m_obj, targ_str) )
+	    AddChildWithText(pgc_node, "button", targ_str);
+    }
     return true;
 }
 
@@ -480,6 +496,13 @@ void IndexVideosForAuthoring()
         GetAuthorNumber(vi) = idx++;
 }
 
+static void AddSubpicStream(xmlpp::Element* sub_node, const char* id, const char* mode)
+{
+    xmlpp::Element* strm_node = sub_node->add_child("stream");
+    strm_node->set_attribute("id", id);
+    strm_node->set_attribute("mode", mode);
+}
+
 void GenerateDVDAuthorScript(const std::string& out_dir)
 {
     // индексируем видео в titles
@@ -558,6 +581,14 @@ void GenerateDVDAuthorScript(const std::string& out_dir)
         // * меню
         xmlpp::Element* menus_node = tts_node->add_child("menus");
         AddVideoTag(menus_node, IsMenuToBe4_3());
+        if( !IsMenuToBe4_3() )
+        {
+            // Jim Taylor, DVD Demustified - для нормального отображения меню 16:9
+            // нужны отдельные субтитры для widescreen и letterbox
+            xmlpp::Element* sub_node  = menus_node->add_child("subpicture");
+            AddSubpicStream(sub_node, "0", "widescreen");
+            AddSubpicStream(sub_node, "1", "letterbox");
+        }
         ForeachMenu(bb::bind(&ScriptMenu, menus_node, root_menu, _1, _2));
         // * список разделов (titles)
         xmlpp::Element* ts_node = tts_node->add_child("titles");
@@ -583,6 +614,7 @@ void ClearTaggedData(Menu mn, const char* tag)
 static bool AuthorClearMenu(Menu mn)
 {
     ClearTaggedData(mn, AUTHOR_TAG);
+    ClearTaggedData(mn, AUTHOR_LB_TAG);
     return true;
 }
 
@@ -634,7 +666,7 @@ struct RestrictGetCanvasBuf
     }
 };
 
-static void CheckHomeSpumuxFont()
+static void CheckSpumuxFontFile()
 {
     char* home_dir = getenv("HOME");
     if( !home_dir )
@@ -653,63 +685,384 @@ static void CheckHomeSpumuxFont()
 std::string FFmpegToDVDTranscode(const std::string& src_fname, const std::string& dst_fname,
                                  const AutoDVDTransData& atd, bool is_pal, const DVDTransData& td)
 {
-    return boost::format("ffmpeg -i %1% %2%") % FilenameForCmd(src_fname) 
-        % FFmpegToDVDArgs(dst_fname, atd, is_pal, td) % bf::stop;
+    return boost::format("%3% -i %1% %2%") % FilenameForCmd(src_fname) 
+        % FFmpegToDVDArgs(dst_fname, atd, is_pal, td) % AVCnvBin() % bf::stop;
 }
 
-// ffmpeg выводит размер первого создаваемого файла каждые полсекунды,
+// ffmpeg выводит статистику первого создаваемого файла каждые полсекунды,
 // см. print_report() (при verbose=1, по умолчанию)
-// Формат: "size=%8.0fkB"
+// Формат размера: "size=%8.0fkB"
 re::pattern FFmpegSizePat( "size= *"RG_NUM"kB"); 
+// Формат длительности: "time=%0.2f"
+re::pattern FFmpegDurPat( "time="RG_FLT);
 
-static void OnTranscodePrintParse(const char* dat, int sz, const io::pos trans_total,
-                                  const io::pos trans_done, const io::pos trans_val)
+typedef boost::function<double(double)> PercentFunctor;
+
+struct Job
+{
+        int  coeff;
+    io::pos  transDone;
+    io::pos  transVal;
+        int  outIdx;
+        
+       GPid  pid;
+  ptr::shared<OutErrBlock> oeb;
+         sigc::connection  watchConn;
+        
+    Job(int coeff_, int out_idx, io::pos trans_val)
+        : coeff(coeff_), transDone(), outIdx(out_idx), transVal(trans_val), pid(NO_HNDL)
+    { ASSERT( coeff > 0 ); }
+};
+
+typedef std::list<Job> JobList;
+
+struct JobData
+{
+        JobList  jLst;
+            int  todoIdx;
+    
+        io::pos  transDone;
+        io::pos  transTotal;
+    std::string  outDir;
+       ExitData  lastED;
+    std::string  exception; // проброс исключений из gtk_main()
+           
+    JobData(): todoIdx(0), transDone(0), transTotal(0) {}
+};
+
+static double CalcTransPercent(double cur_dur, Job& job, JobData& jd, double full_dur)
+{
+    //return (std::min(sz * 1024, trans_val) + trans_done)/double(trans_total);
+    //return (std::min(cur_dur/full_dur, 1.) * trans_val + trans_done)/double(trans_total);
+    job.transDone = std::min(cur_dur/full_dur, 1.) * job.transVal;
+    double res = 0.;
+    boost_foreach( Job& j, jd.jLst )
+        res += j.transDone;
+    return (res + jd.transDone)/double(jd.transTotal);
+}
+
+static void OnTranscodePrintParse(const char* dat, int sz, const PercentFunctor& fnr)
 {
     re::match_results what;
-    if( re::search(std::string(dat, sz), what, FFmpegSizePat) )
+    // лучше вычислять не по выданному размеру, а по времени, так как итоговый размер
+    // вычисляется по битрейту (и обычно завышен для страховки), а длительность практически не
+    // меняется
+    //if( re::search(std::string(dat, sz), what, FFmpegSizePat) )
+    //{
+    //    // :KLUDGE: вроде и нецелые могут быть, но precision=0
+    //    // как-то не в тему
+    //    // У пользователей случалось вообще не число, потому гасим возможные исключения
+    //    io::pos sz;
+    //    if( Str::GetType(sz, what.str(1).c_str()) )
+    //    {
+    //        double per = fnr(sz);
+    //        Author::SetStageProgress(per);
+    //    }
+    //}
+    if( re::search(std::string(dat, sz), what, FFmpegDurPat) )
     {
-        // :KLUDGE: вроде и нецелые могут быть, но precision=0
-        // как-то не в тему
-        // У пользователей случалось вообще не число, потому гасим возможные исключения
-        io::pos sz;
-        if( Str::GetType(sz, what.str(1).c_str()) )
+        double dur;
+        if( ExtractDouble(dur, what) )
         {
-            double per = (std::min(sz * 1024, trans_val) + trans_done)/double(trans_total);
+            double per = fnr(dur);
             Author::SetStageProgress(per);
         }
     }
 }
 
+static void ToCout(const char* dat, int sz, const std::string& prefix)
+{
+    io::cout << prefix << std::string(dat, sz);
+}
+
+ReadReadyFnr DetailsAppender(const std::string& print_cmd, const ReadReadyFnr& add_fnr,
+                             const std::string& prefix)
+{
+    return TextViewAppender(PrintCmdToDetails(print_cmd), add_fnr, prefix);
+}
+
+static void OnJobEnd(GPid pid, int status, JobData& jd);
+static bool UpdateJobs(JobData& jd)
+{
+    int max_wl = Prefs().maxCPUWorkload;
+    // в промежутке [1, max]
+    max_wl = std::min(std::max(1, max_wl), MaxCPUWorkload());
+    
+    JobList& jl = jd.jLst;
+    int cnt = boost::distance(AllTransVideos());
+    // отрисовываем префикс, чтоб разделять выводы; исходя из
+    // приоритета паралелльного выполнения над многопоточностью,
+    // таково условие нужности префикса
+    bool multiple_out = (max_wl > 1) && (cnt > 1);
+        
+    int wl = 0;
+    boost_foreach( Job& job, jl )
+        wl += job.coeff;
+    // пока применяем простую практику - максимальную нагрузку не могут снизить в процессе
+    wl = max_wl - wl;
+    ASSERT( wl >= 0 );
+
+    int new_cnt = std::min(wl, cnt-jd.todoIdx);
+    if( new_cnt > 0 )
+    {
+        int div = wl / new_cnt;
+        int rest  = wl % new_cnt;
+        for( int i=0; i<new_cnt; i++, rest-- )
+        {
+            // мин. неиспользуемое значение
+            int out_idx = 0;
+            for( ; ; out_idx++ )
+            {
+                bool idx_found = true;
+                boost_foreach( Job& job, jl )
+                    if( job.outIdx == out_idx )
+                    {
+                        idx_found = false;
+                        break;
+                    }
+                if( idx_found )
+                    break;
+            }
+            
+            static bool no_ffmpeg_threads = false;
+            static RPData nft_rp;
+            if( ReadPref("no_ffmpeg_threads", nft_rp) )
+                no_ffmpeg_threads = PrefToBool(nft_rp.val);
+            
+            int coeff = rest > 0 ? div+1 : div;
+            if( no_ffmpeg_threads )
+                coeff = 1;
+            VideoItem vi = AllTransVideos().advance_begin(jd.todoIdx+i).front();
+
+            std::string src_fname = SrcFilename(vi);
+            std::string dst_fname = DVDCompliantFilename(vi, jd.outDir);
+            // :TRICKY: отдельные видеофайлы подгоняем под DVD-формат исходя из них самих
+            // (а не под одну "гребенку" проекта), несмотря на то, что запись в один VTS видео
+            // разных форматов вроде как противоречит спекам. Причина: вроде и так работает (на
+            // доступных мне железных и софтовых плейерах), а если будут проблемы - лучше мультиформат
+            // реализовать
+            RTCache& rtc = GetRTC(vi);
+            AutoDVDTransData atd(Is4_3(vi));
+            atd.asd = rtc.asd;
+            atd.asd.audioNum = OutAudioNum(atd.asd);
+            atd.threadsCnt = coeff;
+
+            DVDTransData td = GetRealTransData(vi);
+            td.ctmFFOpt = CustomFFOpts(vi);
+
+            std::string ffmpeg_cmd = FFmpegToDVDTranscode(src_fname, dst_fname, atd, IsPALProject(), td);
+            jl.push_back(Job(coeff, out_idx, CalcTransSize(vi, td.vRate)));
+            Job& new_job = jl.back();
+
+            PercentFunctor fnr = bb::bind(&CalcTransPercent, _1, b::ref(new_job), b::ref(jd), rtc.duration);
+            ReadReadyFnr rr_fnr = bb::bind(&OnTranscodePrintParse, _1, _2, fnr);
+
+            int out_err[2];
+            GPid pid = Spawn(0, ffmpeg_cmd.c_str(), out_err, true);
+            new_job.pid = pid;
+            
+            std::string prefix;
+            if( multiple_out )
+                prefix = boost::format("%1%: ") % out_idx % bf::stop;
+            ReadReadyFnr oeb_fnr;
+            std::string print_ffcmd = prefix + ffmpeg_cmd;
+            if( Execution::ConsoleMode::Flag )
+            {
+                io::cout << print_ffcmd << io::endl;
+                oeb_fnr = bb::bind(&ToCout, _1, _2, prefix);
+            }
+            else
+                oeb_fnr = DetailsAppender(print_ffcmd, rr_fnr, prefix);
+            new_job.oeb = new OutErrBlock(out_err, oeb_fnr);
+
+            new_job.watchConn = Glib::signal_child_watch().connect(bb::bind(&OnJobEnd, _1, _2, b::ref(jd)), pid);
+        }
+        jd.todoIdx += new_cnt;
+    }
+    
+    return !jl.empty();
+}
+
+// если создавали процесс с признаком ожидания, то нужен StopAndWait(), даже
+// если результат процесса не нужен:
+// - Posix: одного Stop() недостаточно,- waitpid()/освобождение ресурсов обязательно;
+//   :TODO: как прекратить сделить, если уже не нужно ничего от процесса
+// - Win32: освобождение хэндла процесса
+void StopAndWait(GPid pid)
+{
+    Execution::Stop(pid);
+    WaitForExit(pid);
+}
+
+// :TRICKY: отвественная функция - халатность недопустима
+// (пропуск исключений наверх)
+static void StopJobPool(JobData& jd) throw()
+{
+    JobList& jl = jd.jLst;
+    // закрываем остальные, неважно что работающие
+    boost_foreach( Job& job, jl )
+    {
+        job.watchConn.disconnect();
+        StopAndWait(job.pid);
+    }
+    // :TRICKY: ошибка _STLPORT_VERSION, __GNUC__ (но не _MSC_VER!): по стандарту clear() должен быть
+    // идентичен erase() (по крайней мере для списка), однако в реальности clear() сначала удаляет объекты,
+    // а затем освобождает сам список; из-за этого в деструкторе Job нельзя пользоваться JobList,-
+    // например, из-за этого просходит падение в CalcTransPercent(), при получении последних строк
+    // вывода от процесса
+    //jl.clear();
+    jl.erase(jl.begin(), jl.end());
+    
+    Gtk::Main::quit();
+}
+
+static void OnJobEnd(GPid pid, int status, JobData& jd)
+{
+    try
+    {
+        bool is_found = false;
+        JobList& jl = jd.jLst;
+        for( JobList::iterator it = jl.begin(), end = jl.end(); it != end; ++it )
+        {
+            Job& job = *it;
+            if( job.pid == pid )
+            {
+                jd.transDone += job.transVal;
+                jl.erase(it);
+                is_found = true;
+                break;
+            }
+        }
+        ASSERT_RTL( is_found );
+
+        jd.lastED = CloseProcData(pid, status);
+
+        if( !jd.lastED.IsGood() )
+            StopJobPool(jd);
+        else if( !UpdateJobs(jd) )
+            Gtk::Main::quit();
+    }
+    catch(const std::exception& exc )
+    {
+        // приходится пробрасывать исключение(я), так как GTK(C) не
+        // не пропускает их через себя
+        jd.exception = exc.what();
+        StopJobPool(jd);
+    }
+}
+
+void RunSpumux(const std::string& xml_fname, const std::string& src_fname, const std::string& dst_fname,
+               int stream_id, const char* dir)
+{
+    CheckSpumuxFontFile();
+    std::string spumux_cmd = boost::format("spumux -m dvd -s%4% %1% < %2% > %3%") 
+        % FilenameForCmd(xml_fname) % src_fname % dst_fname % stream_id % bf::stop;
+    RunExtCmd(spumux_cmd, "spumux", ReadReadyFnr(), dir);
+}
+
+static bool IsAuthorMode(Author::Mode mod)
+{
+    return Author::GetES().mode == mod;
+}
+
+static void AppendOpt(std::string& args, const std::string& opt_name, const std::string& opt_val)
+{
+    if( opt_val.size() )
+        args += " " + opt_name + " " + QuotedName(opt_val);
+}
+
+static void RunAuthorCmd(const std::string& cmd, const char* app_name, 
+                         Author::OutputFilter& of, const std::string& out_dir)
+{
+    if( Execution::ConsoleMode::Flag )
+        RunExtCmd(cmd, app_name, ReadReadyFnr(), out_dir.c_str());
+    else
+    {
+        std::string line = cmd + "\n";
+        of.OnGetLine(line.c_str(), line.size(), true);
+
+        ExitData ed = Author::AsyncOFCall(cmd.c_str(), out_dir.c_str(), of);
+        Author::CheckAppED(ed, app_name);
+    }
+}
+
+static std::string MkisofsArgs()
+{
+    std::string mkisofs_args;
+    AppendOpt(mkisofs_args, "-V", Author::ResDiscLabel());
+    AppendOpt(mkisofs_args, "-dvd-video", "dvd");
+    return mkisofs_args;
+}
+
+void RunBurnCmd(Author::OutputFilter& of, const std::string& out_dir)
+{
+    // -use-the-force-luke=tty - чтобы без сомнений переписывал сожержимое DVD-+RW
+    std::string cmd("growisofs -dvd-compat -use-the-force-luke=tty");
+    double dev_speed = Author::GetBurnerSpeed();
+    AppendOpt(cmd, "-speed", dev_speed ? Double2Str(dev_speed) : "");
+    AppendOpt(cmd, "-Z", Author::BurnerDrive());
+
+    RunAuthorCmd(cmd+MkisofsArgs(), "growisofs", of, out_dir);
+}
+
+bool IsSConsAuthoring()
+{
+    return PrefToBool(PrefContents("script_authoring"));
+}
+
 static void AuthorImpl(const std::string& out_dir)
 {
     AuthorSectionInfo((str::stream() << "Build DVD-Video in folder: " << out_dir).str());
     IteratePendingEvents();
 
     IndexVideosForAuthoring();
+    Author::ExecState& es = Author::GetES();
 
     // * транскодирование
-    io::pos trans_done = 0, trans_total = ProjectStat().transSum;
+    //io::pos trans_done = 0, trans_total = ProjectStat().transSum;
+    //boost_foreach( VideoItem vi, AllTransVideos() )
+    //{
+    //    std::string src_fname = SrcFilename(vi);
+    //    std::string dst_fname = DVDCompliantFilename(vi, out_dir);
+    //    // :TRICKY: отдельные видеофайлы подгоняем под DVD-формат исходя из них самих
+    //    // (а не под одну "гребенку" проекта), несмотря на то, что запись в один VTS видео
+    //    // разных форматов вроде как противоречит спекам. Причина: вроде и так работает (на
+    //    // доступных мне железных и софтовых плейерах), а если будут проблемы - лучше мультиформат
+    //    // реализовать
+    //    RTCache& rtc = GetRTC(vi);
+    //    AutoDVDTransData atd(Is4_3(vi));
+    //    atd.audioNum  = OutAudioNum(rtc.audioNum);
+    //    atd.srcAspect = rtc.dar;
+    //
+    //    DVDTransData td = GetRealTransData(vi);
+    //    td.ctmFFOpt = CustomFFOpts(vi);
+    //
+    //    std::string ffmpeg_cmd = FFmpegToDVDTranscode(src_fname, dst_fname, atd, IsPALProject(), td);
+    //    io::pos trans_val  = CalcTransSize(vi, td.vRate);
+    //    PercentFunctor fnr = bb::bind(&CalcTransPercent, _1, trans_total, trans_done,
+    //                                  trans_val, rtc.duration);
+    //    RunFFmpegCmd(ffmpeg_cmd, bb::bind(&OnTranscodePrintParse, _1, _2, fnr));
+    //
+    //    trans_done += trans_val;
+    //}
     Author::SetStage(Author::stTRANSCODE);
-    boost_foreach( VideoItem vi, AllTransVideos() )
     {
-        std::string src_fname = SrcFilename(vi);
-        std::string dst_fname = DVDCompliantFilename(vi, out_dir);
-        // :TRICKY: отдельные видеофайлы подгоняем под DVD-формат исходя из них самих
-        // (а не под одну "гребенку" проекта), несмотря на то, что запись в один VTS видео
-        // разных форматов вроде как противоречит спекам. Причина: вроде и так работает (на
-        // доступных мне железных и софтовых плейерах), а если будут проблемы - лучше мультиформат
-        // реализовать
-        RTCache& rtc = GetRTC(vi);
-        AutoDVDTransData atd(Is4_3(vi));
-        atd.audioNum  = OutAudioNum(rtc.audioNum);
-        atd.srcAspect = rtc.dar;
-        DVDTransData td = GetRealTransData(vi);
-
-        std::string ffmpeg_cmd = FFmpegToDVDTranscode(src_fname, dst_fname, atd, IsPALProject(), td);
-        io::pos trans_val      = CalcTransSize(rtc, td.vRate);
-        RunFFmpegCmd(ffmpeg_cmd, bb::bind(&OnTranscodePrintParse, _1, _2, trans_total, trans_done, trans_val));
-
-        trans_done += trans_val;
+        JobData jd;
+        jd.transTotal = ProjectStat().transSum;
+        jd.outDir     = out_dir;
+        
+        if( UpdateJobs(jd) )
+        {
+            ActionFunctor& stop_fnr = es.eDat.stopFnr;
+            stop_fnr = bb::bind(&StopJobPool, b::ref(jd));
+            Gtk::Main::run();
+            stop_fnr.clear();
+            
+            if( jd.exception.size() )
+                Author::Error(jd.exception);
+            Author::CheckAbortByUser();
+            Author::CheckAppED(jd.lastED, AVCnvBin());
+        }
     }
 
     // * субтитры
@@ -740,7 +1093,7 @@ static void AuthorImpl(const std::string& out_dir)
             // не любит ничего кроме 720xfull и 28.0,- 
             // решил в пользу софтовых ради удобства разработки, но лучше бы
             // разобраться (хорошо, что не 720xfull - редкость)
-            DVDDims dd = RequireTranscoding(vi) ? GetRealTransData(vi).dd : CalcDimsAuto(vi);
+            DVDDims dd = RequireVideoTC(vi) ? GetRealTransData(vi).dd : CalcDimsAuto(vi);
             // чуть меньше делаем размеры
             Point movie_sz = DVDDimension(dd) - Point(2, 2);
             ts->set_attribute("movie-width",  Int2Str(movie_sz.x));
@@ -768,14 +1121,9 @@ static void AuthorImpl(const std::string& out_dir)
             std::string xml_fname = PrefixCnvPath(vi, out_dir) + ".textsub.xml";
             SaveFormattedUTF8Xml(doc, xml_fname);
             
-            CheckHomeSpumuxFont();
-            std::string spumux_cmd = boost::format("spumux -m dvd %1% < %2% > %3%") 
-                % FilenameForCmd(xml_fname) % FilenameForCmd(DVDCompliantFilename(vi, out_dir))
-                % FilenameForCmd(DVDFilename(vi, out_dir)) % bf::stop;
-            RunExtCmd(spumux_cmd, "spumux");
+            RunSpumux(xml_fname, DVDCompliantFilename(vi, out_dir), DVDFilename(vi, out_dir));
         }
 
-    Author::ExecState& es = Author::GetES();
     str::stream& settings = es.settings;
     settings.str("# coding: utf-8\n\n");
     // * рендерим меню и их скрипты
@@ -785,7 +1133,8 @@ static void AuthorImpl(const std::string& out_dir)
     GenerateDVDAuthorScript(out_dir);
 
     // * вспомогательные файлы
-    const char* fnames[] = { "SConstruct", "ADVD.py", "SConsTwin.py", "README" };
+    //const char* fnames[] = { "SConstruct", "ADVD.py", "SConsTwin.py", "README" };
+    const char* fnames[] = { "SConstruct", "SConsTwin.py", "README" };
     for( int i=0; i<(int)ARR_SIZE(fnames); i++ )
         CopyRootFile(fnames[i], out_dir);
     // опции
@@ -797,19 +1146,30 @@ static void AuthorImpl(const std::string& out_dir)
     settings_strm.close();
 
     // *
-    if( !Execution::ConsoleMode::Flag )
+    if( !IsAuthorMode(Author::modRENDERING) )
     {
-        if( es.mode != Author::modRENDERING )
+        if( IsSConsAuthoring() )
+        {
+            if( !Execution::ConsoleMode::Flag )
+            {
+                Author::BuildDvdOF of;
+                Author::ExecuteSconsCmd(out_dir, of, es.mode, scons_options);
+            }
+        }
+        else
         {
             Author::BuildDvdOF of;
-            Author::ExecuteSconsCmd(out_dir, of, es.mode, scons_options);
+            RunAuthorCmd("dvdauthor -o dvd -x DVDAuthor.xml", "dvdauthor", of, out_dir);
+
+            if( IsAuthorMode(Author::modDISK_IMAGE) )
+            {
+                std::string cmd = MK_ISO_CMD + MkisofsArgs() + " > dvd.iso";
+                RunAuthorCmd(cmd, MK_ISO_CMD, of, out_dir);
+            }
+            else if( IsAuthorMode(Author::modBURN) )
+                RunBurnCmd(of, out_dir);
         }
     }
-    else
-    {
-        ////int pid = gnome_execute_shell(out_dir.c_str(), "scons totem");
-        //int pid = Spawn(out_dir.c_str(), "scons totem");
-    }
 }
 
 std::string AuthorDVD(const std::string& out_dir)
@@ -879,40 +1239,62 @@ std::string SconsTarget(Mode mode)
     return res;
 }
 
+static std::string ReplaceChar(const std::string& label, char old_char, char new_char)
+{
+    std::string clean_label;
+    for( int i=0; i<(int)label.size(); i++ )
+        clean_label += (label[i] != old_char) ? label[i] : new_char;
+    return clean_label;
+}
+
 // for_cmd - для командной строки/для скрипта ASettings.py
 static void AddSconsOptions(str::stream& strm, bool for_cmd, const std::string& dvd_label, 
                             const std::string& dvd_drive, double dvd_speed)
 {
-    // :TODO: не знаю как передавать символы ' и ", и в командную строку,
-    // и в скрипт
-    std::string clean_label;
-    for( int i=0; i<(int)dvd_label.size(); i++ )
-        clean_label += (dvd_label[i] != '\'') ? dvd_label[i] : '`';
-
     char sep = for_cmd ? ' ' : '\n' ;
     if( for_cmd )
         strm << sep;
-    strm << "DVDLabel='" << clean_label << "'" << sep;
-    strm << "DVDDrive='" << dvd_drive   << "'" << sep;
+    strm << "DVDLabel=\"" << dvd_label << "\"" << sep;
+    strm << "DVDDrive=\"" << dvd_drive   << "\"" << sep;
     strm << "DVDSpeed="  << dvd_speed;
     if( !for_cmd )
         strm << sep << sep;
 }
 
+std::string DiscLabel()
+{
+    return GetBD().Label().get_text();
+}
+
+std::string ResDiscLabel()
+{
+    std::string res_label;
+    if( !Execution::ConsoleMode::Flag )
+    {
+        // :TODO: не знаю как передавать символы ' и ", и в командную строку,
+        // и в скрипт
+        // :TODO2: под Win32 буква апостроф "'" не воспринимается как разделитель слов,
+        // потому заменяем " => ' (потому что " - служебный)
+        //std::string clean_label = ReplaceChar(DiscLabel(), '\'', '`');
+        res_label = ReplaceChar(DiscLabel(), '"', '\'');
+    }
+    return res_label;
+}
+
 void FillSconsOptions(str::stream& scons_options, bool fill_def)
 {
-    std::string def_dvd_label, def_drive;
+    std::string dvd_label = ResDiscLabel();
+
+    std::string def_drive;
     double def_speed = 0;
     if( !Execution::ConsoleMode::Flag )
     {     
-        BurnData& bd = GetBD();
-        def_dvd_label = bd.Label().get_text();
         if( IsBurnerSetup(def_drive) )
             def_speed = GetBurnerSpeed();
     }
     if( fill_def )
-        AddSconsOptions(GetES().settings, false, def_dvd_label, def_drive, def_speed);
-    AddSconsOptions(scons_options, true, def_dvd_label, def_drive, def_speed);
+        AddSconsOptions(GetES().settings, false, dvd_label, def_drive, def_speed);
+    AddSconsOptions(scons_options, true, dvd_label, def_drive, def_speed);
 }
 
 void CheckAbortByUser()
@@ -921,6 +1303,12 @@ void CheckAbortByUser()
         throw std::runtime_error("User Abortion"); // строка реально не нужна - сработает userAbort
 }
 
+void CheckAppED(const ExitData& ed, const char* app_name)
+{
+    if( !ed.IsGood() )
+        Author::ApplicationError(app_name, ed);
+}
+
 ExitData AsyncCall(const char* dir, const char* cmd, const ReadReadyFnr& fnr)
 {
     ExitData ed = ExecuteAsync(dir, cmd, fnr, &GetES().eDat.pid);
@@ -949,11 +1337,31 @@ void ApplicationError(const char* app_name, const ExitData& ed)
     ApplicationError(app_name, ExitDescription(ed));
 }
 
+std::string GetSConsName()
+{
+#ifdef _WIN32
+    //std::string cmd("scons.bat");
+    // можно и без .exe, но раз единообразно нельзя, то конкретизируем
+    std::string cmd("scons.bin.exe");
+#else
+    std::string cmd("scons");
+#endif
+    return cmd;
+}
+
+ExitData AsyncOFCall(const std::string& cmd, const std::string& out_dir, OutputFilter& of)
+{
+    ExitData ed = AsyncCall(out_dir.c_str(), cmd.c_str(), OF2RRF(of));
+    if( of.firstError.size() )
+        Error(of.firstError);
+    return ed;
+}
+
 void ExecuteSconsCmd(const std::string& out_dir, OutputFilter& of, 
                      Mode mod, const str::stream& scons_options)
 {
-    std::string cmd = "scons" + scons_options.str() + " " + SconsTarget(mod);
-    ExitData ed = AsyncCall(out_dir.c_str(), cmd.c_str(), OF2RRF(of));
+    std::string cmd = GetSConsName() + scons_options.str() + " " + SconsTarget(mod);
+    ExitData ed = AsyncOFCall(cmd, out_dir, of);
     if( !ed.IsGood() )
         //ApplicationError("", ed);
         Error(BF_("external command failure: %1%") % ExitDescription(ed) % bf::stop);
