@@ -34,6 +34,7 @@
 #include <mgui/sdk/entry.h>
 #include <mgui/sdk/packing.h>
 #include <mgui/sdk/widget.h>
+#include <mgui/sdk/treemodel.h>
 
 #include <mgui/render/menu.h>
 #include <mgui/render/editor.h>
@@ -110,22 +111,121 @@ Gtk::Widget& MakeSelectionToolImage()
     return GetFactoryGtkImage("tool-pointer.png");
 }
 
-Gtk::TreeModelColumn<std::string> FrameTypeColumn;
-
-static RefPtr<Gdk::Pixbuf> RenderToolbarFrame(const std::string& theme_str, uint fill_clr)
+static RefPtr<Gdk::Pixbuf> ToToolbarSize(RefPtr<Gdk::Pixbuf> frame_pix)
 {
-    const Editor::ThemeData& td   = Editor::GetThumbTheme(theme_str);
-    Point thumb_sz(PixbufSize(td.vFrameImg));
-    RefPtr<Gdk::Pixbuf> frame_pix = CreatePixbuf(thumb_sz);
-    frame_pix->fill(fill_clr);
-    frame_pix = Editor::CompositeWithFrame(frame_pix, td);
-
+    Point thumb_sz(PixbufSize(frame_pix));
     Point sz(Round(thumb_sz.x*Editor::TOOL_IMAGE_SIZE/(double)thumb_sz.y), Editor::TOOL_IMAGE_SIZE);
     RefPtr<Gdk::Pixbuf> pix = CreatePixbuf(sz);
     RGBA::Scale(pix, frame_pix);
     return pix;
 }
 
+static RefPtr<Gdk::Pixbuf> RenderToolbarFrame(const std::string& theme_str, uint fill_clr)
+{
+    const Editor::ThemeData& td = Editor::GetThumbTheme(theme_str);
+    RefPtr<Gdk::Pixbuf> frame_pix = CreatePixbuf(PixbufSize(td.vFrameImg));
+    frame_pix->fill(fill_clr);
+    frame_pix = Editor::CompositeWithFrame(frame_pix, td);
+
+    return ToToolbarSize(frame_pix);
+}
+
+typedef Gtk::TreeModelColumn<FrameTheme> FTColumn;
+
+struct FTFields
+{
+    FTColumn  ft_cln;
+
+    FTFields(Gtk::TreeModelColumnRecord& rec)
+    {
+        rec.add(ft_cln);
+    }
+};
+
+FTColumn& FrameTypeColumn()
+{
+    return GetColumnFields<FTFields>().ft_cln;
+}
+
+// значение итератора из списка (combobox) тем
+FrameTheme Iter2FT(const Gtk::TreeModel::iterator& iter)
+{
+    return iter->get_value(FrameTypeColumn());
+}
+
+std::string FT2PrintName(const FrameTheme& ft)
+{
+    std::string res = ft.themeName;
+    if( ft.isIcon )
+        res = get_basename(res);
+    return res;
+}
+
+static std::string FT2Name(const Gtk::TreeModel::iterator& iter)
+{
+    FrameTheme ft = Iter2FT(iter);
+    return FT2PrintName(ft);
+}
+
+bool SeparatorFunc(const RefPtr<Gtk::TreeModel>&, const Gtk::TreeIter& itr)
+{
+    return FT2Name(itr) == "_separator_";
+}
+
+Str::List& IconsDirs()
+{
+    static Str::List dirs;
+    if( !dirs.size() )
+        Project::AddSrcDirs(dirs, "icons");
+    return dirs;
+}
+
+typedef std::pair<fs::directory_iterator, fs::directory_iterator> dir_range_t;
+
+dir_range_t dir_range(const fs::path& dir)
+{
+    return dir_range_t(fs::directory_iterator(dir), fs::directory_iterator());
+}
+
+static bool ByExtName(const std::string& e1, const std::string& e2)
+{
+    bool cmp_res;
+    if( CompareComponent(get_extension(e1), get_extension(e2), cmp_res) ||
+	CompareComponent(e1, e2, cmp_res) )
+	return cmp_res;
+
+    return false;
+}
+
+RefPtr<Gdk::Pixbuf> LoadThemeIconTry(const std::string& theme_fname, const Point& sz,
+                                  bool preserve_aspect)
+{
+    RefPtr<Gdk::Pixbuf> icon_pix;
+    boost_foreach( const std::string& str, IconsDirs() )
+    {
+        fs::path pth = fs::path(str) / theme_fname;
+        if( fs::exists(pth) )
+            try
+            {
+                icon_pix = Gdk::Pixbuf::create_from_file(pth.string(), sz.x, sz.y, preserve_aspect);
+            } 
+            catch(...) {}
+
+        if( icon_pix )
+            break;
+    }
+    return icon_pix;
+}
+
+RefPtr<Gdk::Pixbuf> LoadThemeIcon(const std::string& theme_fname, const Point& sz,
+                                  bool preserve_aspect)
+{
+    RefPtr<Gdk::Pixbuf> pix = LoadThemeIconTry(theme_fname, sz, preserve_aspect);
+    // :TODO: рисовать что-то вопросительное при отсутствии надо
+    ASSERT_RTL( pix );
+    return pix;
+}
+
 Toolbar::Toolbar(): selTool(MakeSelectionToolImage()), txtTool(MakeTextToolLabel())
     // внучную построим кнопки
     //, bldBtn(Gtk::Stock::BOLD), itaBtn(Gtk::Stock::ITALIC), undBtn(Gtk::Stock::UNDERLINE)
@@ -141,16 +241,6 @@ Toolbar::Toolbar(): selTool(MakeSelectionToolImage()), txtTool(MakeTextToolLabel
 
     // * выбор рамки
     {
-        // * создаем модель
-        Gtk::TreeModelColumnRecord columns;
-        Gtk::TreeModelColumn<RefPtr<Gdk::Pixbuf> > pix_cln;
-        Gtk::TreeModelColumn<std::string>          str_cln;
-        columns.add(pix_cln);
-        columns.add(str_cln);
-        // добавляем str_cln, чтобы можно было много раз создавать панель инструментов
-        Editor::FrameTypeColumn = str_cln;
-        RefPtr<Gtk::ListStore> f_store = Gtk::ListStore::create(columns);
-
         // * нужен цвет фона
         unsigned int fill_clr = 0;
         {
@@ -159,8 +249,14 @@ Toolbar::Toolbar(): selTool(MakeSelectionToolImage()), txtTool(MakeTextToolLabel
             tmp_win.add(tmp_combo);
             fill_clr = GetStyleColorBg(Gtk::STATE_NORMAL, tmp_combo).ToUint();
         }
-        // * заполняем
-        Gtk::ComboBox& combo = frame_combo;
+    	// * создаем модель
+    	Gtk::TreeModelColumn<RefPtr<Gdk::Pixbuf> > pix_cln;
+    	Gtk::TreeModelColumnRecord& columns = GetColumnRecord<FTFields>();
+    	columns.add(pix_cln);
+    	RefPtr<Gtk::ListStore> f_store = Gtk::ListStore::create(columns);
+    
+    	// * заполняем
+    	Gtk::ComboBox& combo = frame_combo;
         combo.set_model(f_store);
         // по фокусу определяем источник изменений!
         //combo.set_focus_on_click(false);
@@ -174,18 +270,52 @@ Toolbar::Toolbar(): selTool(MakeSelectionToolImage()), txtTool(MakeTextToolLabel
             const std::string& theme_str = iv.value();
             // *
             row[pix_cln] = RenderToolbarFrame(theme_str, fill_clr);
-            row[str_cln] = theme_str;
+            row[FrameTypeColumn()] = theme_str;
             if( theme_str == "rect" )
                 combo.set_active(iv.index());
         }
+	
+    	// * иконки
+    	// фиктивная тема для разделителя
+    	f_store->append()->set_value(FrameTypeColumn(), FrameTheme("_separator_"));
+    	combo.set_row_separator_func(&SeparatorFunc);
+    	// набираем список объектов
+    	Str::List o_lst;
+    	boost_foreach( const std::string& str, IconsDirs() )
+    	{
+    	    fs::path dir = fs::path(str);
+    	    if( fs::is_directory(dir) )
+    	    {    
+    		boost_foreach( const fs::path& pth, dir_range(dir) )
+    		    o_lst.push_back(pth.filename());
+    	    }
+    	}
+    	std::sort(o_lst.begin(), o_lst.end(), &ByExtName);
+        Project::MakeUnique(o_lst);
+        
+    	boost_foreach( const std::string& fname, o_lst )
+    	{
+    	    RefPtr<Gdk::Pixbuf> icon_pix = LoadThemeIconTry(fname, Point(-1, Editor::TOOL_IMAGE_SIZE), true);
+            if( icon_pix )
+            {
+                Gtk::TreeRow row = *f_store->append();
+                row[pix_cln] = icon_pix; // ToToolbarSize(icon_pix);
+                row[FrameTypeColumn()] = FrameTheme(fname, true);
+            }
+    	}
+    	
         // * внешний вид
         combo.pack_start(pix_cln, false);
-        //combo.pack_start(str_cln, true);
         Gtk::CellRendererText& txt_rndr = NewManaged<Gtk::CellRendererText>();
         //txt_rndr.property_size_points() = 12.0;
         txt_rndr.property_xpad() = 5;
         combo.pack_start(txt_rndr, true);
-        combo.add_attribute(txt_rndr, "text", str_cln);
+        //combo.add_attribute(txt_rndr, "text", str_cln);
+    	combo.set_cell_data_func(txt_rndr, bb::bind(&Project::RenderField, &txt_rndr, _1, &FT2Name));
+    	// 1) wrap_width>0 форсирует режим "не списка", что нестандартно для Windows (ерунда, кнопка все равно
+    	// особая)
+    	// 2) в 4 колонки тоже ничего (стараемся принять минимальный размер "в диаметре")
+    	combo.set_wrap_width(3);
     }
 }
 
@@ -212,11 +342,11 @@ TextStyle Toolbar::GetFontDesc()
     return TextStyle(fnt_desc, undBtn.get_active(), GetColor(clrBtn));
 }
 
-std::string GetActiveTheme()
+FrameTheme GetActiveTheme()
 {
     Gtk::ComboBox& combo = MenuToolbar().frame_combo;
     Gtk::TreeIter itr = combo.get_active();
-    return itr ? itr->get_value(FrameTypeColumn) : std::string() ;
+    return itr ? Iter2FT(itr) : FrameTheme() ;
 }
 
 static void AppendTSeparator(Gtk::Toolbar& tbar)
@@ -325,7 +455,7 @@ Rect FindNewObjectLocation(RectMatrix& matr, MenuRegion& mr)
     }
     if( lct.IsNull() ) // не нашли свобоодного места
     {
-        Point sz(mr.GetParams().Size());
+        Point sz(MenuSize(mr));
         Point obj_sz = CalcStandardObjectSize(sz);
         Point a      = FindAForCenteredRect(obj_sz, Rect0Sz(sz));
 
@@ -357,11 +487,29 @@ RectMatrix MakeAddObjectMatrix(const MenuParams& mp)
     return matr;
 }
 
-static void AddFTOItem(MEditorArea& editor, const Rect& lct, Project::MediaItem mi)
+static void AddFTOItem(MEditorArea& editor, const Rect& src_lct, Project::MediaItem mi)
 {
     MenuRegion& rgn = editor.CurMenuRegion();
-
-    FrameThemeObj* fto = Project::NewFTO(Editor::GetActiveTheme(), lct);
+    FrameTheme ft = Editor::GetActiveTheme();
+    Rect lct(src_lct);
+    if( ft.isIcon )
+    {
+        // :KLUDGE: помимо пропорций было бы удобно подходящий размер иконок задавать:
+        // - сейчас вписываем в рамку с размерами REL_OBJ_SZ, уменьшить?
+        // - при DnD-создании меняется форма src_lct, в зависимости от пропорций источника,
+        //   что влияет на конечный размер (хочется избегать зависимости размеров от вида источника?)
+        
+        // вписываем иконку с сохранением(!) пропорций
+        Point dar(PixbufSize(LoadThemeIcon(ft.themeName, Point(-1, -1), true)));
+        // узнаем реальные пропорции lct
+        Point lct_sz(lct.Size());
+        Point menu_dar = rgn.GetParams().DisplayAspect();
+        Point menu_sz  = MenuSize(rgn);
+        Point lct_dar(lct_sz.x * menu_dar.x * menu_sz.y, lct_sz.y * menu_dar.y * menu_sz.x);
+        
+        lct = FitIntoRect(lct_sz, lct_dar, dar) + lct.A();
+    }
+    FrameThemeObj* fto = Project::NewFTO(ft, lct);
     fto->MediaItem().SetLink(mi);
     Project::AddMenuItem(rgn, fto);
 
@@ -372,7 +520,7 @@ void AddFTOItem(MEditorArea& editor, const Point& center, Project::MediaItem mi)
 {
     // :KLUDGE: не самый правильный (и точный) способ сосчитать пропорцию
     Point proportion = editor.Transition().RelToAbs(Project::CalcThumbSize(mi));
-    Point sz(editor.CurMenuRegion().GetParams().Size());
+    Point sz(MenuSize(editor.CurMenuRegion()));
 
     Point it_sz = Project::CalcProportionSize(proportion, sz.x*sz.y*REL_OBJ_SZ*REL_OBJ_SZ);
     Point a(center.x-it_sz.x/2, center.y-it_sz.y/2);
@@ -398,12 +546,12 @@ static void AddObjectClicked()
 
 static bool ReframeFTO(RectListRgn& r_lst, FrameThemeObj* obj, MenuRegion& mn_rgn)
 {
-    const std::string& theme = Editor::GetActiveTheme();
+    const FrameTheme& theme = Editor::GetActiveTheme();
     if( theme != obj->Theme() )
     {
-        obj->GetData<FTOInterPixData>().ClearPix();
+        ClearFTOCache(*obj);
         obj->Theme() = theme;
-        r_lst.push_back( Planed::AbsToRel(mn_rgn.Transition(), obj->Placement()) );
+        AddRelPos(r_lst, obj, mn_rgn.Transition());
     }
     return true;
 }
@@ -433,9 +581,9 @@ static void FontNameChanged(MEditorArea& editor, bool only_clr)
     SetSelObjectsTStyle(editor, editor.Toolbar().GetFontDesc(), only_clr);
 }
 
-static void SetupStyleBtn(Gtk::ToggleButton& btn, Gtk::Toolbar& tbar,
-                          const char* tooltip, const ActionFunctor& fnr,
-                          Gtk::BuiltinStockID stock_id)
+static void SetupToggleBtn(Gtk::ToggleButton& btn, Gtk::Toolbar& tbar,
+                           const char* tooltip, const ActionFunctor& fnr,
+                           Gtk::Widget& img_wdg)
 {
     // вручную строим кнопки а-ля Gtk::ToggleToolButton, потому что
     // нам надо установить у них свойство focus-on-click для последующего
@@ -446,12 +594,19 @@ static void SetupStyleBtn(Gtk::ToggleButton& btn, Gtk::Toolbar& tbar,
     ASSERT( btn.get_focus_on_click() ); // должен быть по умолчанию
     AppendToToolbar(tbar, btn);
     btn.set_relief(Gtk::RELIEF_NONE);
-    btn.add(NewManaged<Gtk::Image>(stock_id, tbar.get_icon_size()));
+    btn.add(img_wdg);
     SetTip(btn, tooltip);
 
     btn.signal_toggled().connect(fnr);
 }
 
+static void SetupStyleBtn(Gtk::ToggleButton& btn, Gtk::Toolbar& tbar,
+                          const char* tooltip, const ActionFunctor& fnr,
+                          Gtk::BuiltinStockID stock_id)
+{
+    SetupToggleBtn(btn, tbar, tooltip, fnr, NewManaged<Gtk::Image>(stock_id, tbar.get_icon_size()));
+}
+
 // хочется отрабатывать изменения в управляющих элементах только когда
 // фокус не на редакторе (т.е. когда не сам редактор изменился)
 static void OnToolbarControlled(const ActionFunctor& fnr)
@@ -475,6 +630,19 @@ static ActionFunctor MakeToolbarFunctor(const ActionFunctor& fnr)
     return bb::bind(&OnToolbarControlled, fnr);
 }
 
+void OnViewChanged(const DrawerFnr& fnr)
+{
+    MEditorArea& edt_area = MenuEditor();
+    CalcRgnForRedraw(edt_area, fnr);
+}
+
+static void SetupViewBtn(Gtk::ToggleButton& btn, Gtk::Toolbar& tbar, const char* tooltip,
+                         const char* icon_fname, const DrawerFnr& fnr)
+{
+    SetupToggleBtn(btn, tbar, tooltip, MakeToolbarFunctor(bb::bind(&OnViewChanged, fnr)),
+                   GetFactoryGtkImage(icon_fname));
+}
+
 Gtk::Toolbar& PackToolbar(MEditorArea& editor, Gtk::VBox& lct_box)
 {
     Editor::Toolbar& edt_tbar = editor.Toolbar();
@@ -558,14 +726,11 @@ Gtk::Toolbar& PackToolbar(MEditorArea& editor, Gtk::VBox& lct_box)
 
     }
 
+    AppendTSeparator(tbar);
     // * кнопка рамки
-    Gtk::ToggleButton& frm_btn = edt_tbar.frmBtn;
-    AppendToToolbar(tbar, frm_btn);
-    frm_btn.set_relief(Gtk::RELIEF_NONE);
-    frm_btn.add(GetFactoryGtkImage("copy-n-paste/lpetool_show_bbox.png"));
-    SetTip(frm_btn, _("Show Safe Area"));
-    // обвязка здесь используется для проверки, что в редакторе есть меню (и не валится при его отстутствии)
-    frm_btn.signal_toggled().connect(MakeToolbarFunctor(&ToggleSafeArea));
+    SetupViewBtn(edt_tbar.frmBtn, tbar, _("Show Safe Area"), "copy-n-paste/lpetool_show_bbox.png", &DrawSafeArea);
+    // *
+    SetupViewBtn(edt_tbar.gridBtn, tbar, _("Snap To Grid"), "copy-n-paste/grid_xy_simple.png", &DrawGrid);
 
     return tbar;
 }
