@@ -26,11 +26,31 @@
 #include "render/common.h" // FillEmpty()
 #include "prefs.h"
 
+#include <mdemux/dvdread.h>
+
 #include <mlib/gettext.h>
 #include <mlib/read_stream.h> // ReadAllStream()
 #include <mlib/string.h>
 
-/////////////////////////////////////////
+#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(54,00,00)
+#define AVFORMAT_54
+#endif
+
+#ifdef AVFORMAT_54
+C_LINKAGE_BEGIN
+#include <libavformat/url.h> // ffurl_register_protocol()
+C_LINKAGE_END
+#endif
+
+// Прямой доступ к ff_codec_bmp_tags, в частности, закрыл, некий
+// Anton Khirnov, см. libavformat/libavformat.v (из него генерится скрипт
+// для опции --version-script=<script_file> линковщика ld)
+// Вообще, можно воспользоваться av_codec_get_tag(), а доступ к ff_codec_bmp_tags
+// получить через ff_avi_muxer->codec_tag (самого же найти по имени "avi") или подобный объект,
+// но это сейчас неактуально (никто не попадается на ошибку отсутствия кодека) => игра не стоит свеч
+//#define CALC_FF_TAG
+
+#ifdef CALC_FF_TAG
 // :KLUDGE: потому что riff.h не копируют
 C_LINKAGE_BEGIN
 
@@ -60,7 +80,7 @@ static uint FFCodecID2Tag(CodecID codec_id)
 #endif
 
 C_LINKAGE_END
-/////////////////////////////////////////
+#endif // CALC_FF_TAG
 
 static AVStream* VideoStream(FFData& ffv)
 {
@@ -212,7 +232,11 @@ void CloseInfo(FFData& ffi)
         // судя по тому как, например, поле ctx_flags нигде не обнуляется
         // (кроме как при инициализации), то повторно использовать структуру
         // не принято -> все заново создаем при переоткрытии
+#ifdef AVFORMAT_54
+        avformat_close_input(&ffi.iCtx);
+#else
         av_close_input_file(ffi.iCtx);
+#endif
         ffi.iCtx = 0;
     }
 }
@@ -258,7 +282,11 @@ static void DumpIFile(AVFormatContext* ic, int idx = 0, const std::string& fname
     //const char* fname = "n/a";
     // входной/выходной файл
     int is_output = 0;
+#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52,101,00)
+    av_dump_format(ic, idx, fname.c_str(), is_output);
+#else
     dump_format(ic, idx, fname.c_str(), is_output);
+#endif
 }
 
 Point VideoSize(AVCodecContext* dec)
@@ -266,11 +294,8 @@ Point VideoSize(AVCodecContext* dec)
     return Point(dec->width, dec->height);
 }
 
-static bool SeekCall(AVFormatContext* ic, int64_t ts, bool is_byte_seek)
+static bool SeekCall(AVFormatContext* ic, int64_t ts, int flags)
 {
-    int flags = is_byte_seek ? AVSEEK_FLAG_BYTE
-        : AVSEEK_FLAG_BACKWARD; // чтоб раньше времени пришли
-
     // вполне подойдет поиск по умолчальному потоку (все равно видео выберут)
     int av_res = av_seek_frame(ic, -1, ts, flags);
     return av_res == 0;
@@ -281,11 +306,6 @@ static bool IsFFError(int av_res)
     return av_res < 0;
 }
 
-static unsigned char GetChar(uint tag, int bit_begin)
-{
-    return (tag>>bit_begin) & 0xFF;
-}
-
 static bool SetIndex(int& idx, int i, bool b)
 {
     bool res = (idx == -1) && b;
@@ -294,8 +314,41 @@ static bool SetIndex(int& idx, int i, bool b)
     return res;
 }
 
-bool OpenInfo(FFData& ffi, const char* fname, std::string& err_str)
+#ifdef CALC_FF_TAG
+static unsigned char GetChar(uint tag, int bit_begin)
 {
+    return (tag>>bit_begin) & 0xFF;
+}
+
+static std::string CodecID2Str(CodecID codec_id)
+{
+#ifdef _MSC_VER
+    std::string tag_str = boost::format("%1%") % codec_id % bf::stop;
+#else // _MSC_VER
+    uint tag = FFCodecID2Tag(codec_id);
+    std::string tag_str = boost::format("0x%1$04x") % tag % bf::stop;
+    unsigned char c0 = GetChar(tag, 0), c8 = GetChar(tag, 8), 
+        c16 = GetChar(tag, 16), c24 = GetChar(tag, 24);
+    if( isprint(c0) && isprint(c8) && isprint(c16) && isprint(c24) )
+        tag_str = boost::format("%1%%2%%3%%4% / %5%") 
+            % c0 % c8 % c16 % c24 % tag_str % bf::stop;
+#endif // !_MSC_VER
+    return tag_str;
+}
+
+#else // CALC_FF_TAG
+
+static std::string CodecID2Str(CodecID codec_id)
+{
+    return Int2Str(codec_id);
+}
+
+#endif // CALC_FF_TAG
+
+bool OpenInfo(FFData& ffi, const char* fname, FFDiagnosis& diag)
+{
+    std::string& err_str = diag.errStr;
+
     av_register_all();
 
     ASSERT( !ffi.IsOpened() );
@@ -305,28 +358,38 @@ bool OpenInfo(FFData& ffi, const char* fname, std::string& err_str)
     // создается из av_find_input_format(str), где str из опции -f для ffmpeg
     // (ffmpeg -formats)
     AVInputFormat* file_iformat = 0;
+
+    AVFormatContext* ic = 0;
+#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(53,2,0)
+    int av_res = avformat_open_input(&ic, fname, file_iformat, 0);
+#else
     // для уточнения параметров входного потока; используется в случаях, когда
     // по самому потоку невозможно определить их (не для контейнеров, а для 
     // элементарных потоков
     AVFormatParameters* ap = 0;
     // всегда нуль (ffmpeg, ffplay)
     int buf_size = 0;
-
-    AVFormatContext* ic = 0;
     int av_res = av_open_input_file(&ic, fname, file_iformat, buf_size, ap);
+#endif
     if( IsFFError(av_res) ) // ошибка
     {
         switch( av_res )
         {
-        case AVERROR_NOENT:
+        case AVERROR(ENOENT):
             // :TODO: решить, ставить в конце точки или нет (сообщения пользователю
             // показывается не HIG-ого)
             err_str = _("No such file");
             break;
-        case AVERROR_NOFMT:
+        case AVERROR(EILSEQ): // использовалось до 0.6, заменено на AVERROR_INVALIDDATA
+        // основной источник ошибок, ffmpeg'у предлагается открыть явно не видео (вроде текста),
+        // см. код ff_probe_input_buffer()
+        // :TODO: однако данный код выбрасывается ffmpeg'ом еще в куче мест; если будет
+        // прецендент, когда эта ошибка выбросится не ff_probe_input_buffer() и дезориентирует
+        // пользователя, то стоит добавить дополнительную проверку явным вызовом
+        // той самой ff_probe_input_buffer()
+        case AVERROR_INVALIDDATA:
             err_str = _("Unknown file format");
             break;
-        case AVERROR_UNKNOWN:
         default:
             err_str = boost::format("FFmpeg unknown error: %1%") % av_res % bf::stop;
             break;
@@ -339,7 +402,11 @@ bool OpenInfo(FFData& ffi, const char* fname, std::string& err_str)
         //
         ffi.iCtx = ic;
     
+#ifdef AVFORMAT_54
+        av_res = avformat_find_stream_info(ic, 0);
+#else
         av_res = av_find_stream_info(ic);
+#endif
         if( IsFFError(av_res) )
         {
             // например .webm для FFmpeg <= 0.5 
@@ -354,13 +421,13 @@ bool OpenInfo(FFData& ffi, const char* fname, std::string& err_str)
         {
             AVStream* strm = ic->streams[i];
             AVCodecContext* avctx = strm->codec;
-            if( SetIndex(video_idx, i, avctx->codec_type == CODEC_TYPE_VIDEO) )
+            if( SetIndex(video_idx, i, avctx->codec_type == AVMEDIA_TYPE_VIDEO) )
                 ;
             else
                 // для демиксера имеет значение только NONE и ALL
                 strm->discard = AVDISCARD_ALL;
 
-            SetIndex(audio_idx, i, avctx->codec_type == CODEC_TYPE_AUDIO);
+            SetIndex(audio_idx, i, avctx->codec_type == AVMEDIA_TYPE_AUDIO);
         }
 
         if( video_idx == -1 )
@@ -386,10 +453,20 @@ bool OpenInfo(FFData& ffi, const char* fname, std::string& err_str)
             // в 99% отсутствие нач. времени - элементарный поток = без контейнера;
             // см. особенности ffmpeg, update_initial_timestamps()
             err_str = _("Start time of the file is unknown");
+            diag.isElemStream = true;
             return false;
         }
 
-        if( !SeekCall(ic, ic->start_time, false) )
+        // :TRICKY: индекс Duck_Dodgers_101a_Duck_Deception_[Moonsong].avi таков,
+        // что в начало не прыгнуть (первое значение > start_time), потому
+        // AVSEEK_FLAG_BACKWARD дает неудачу, хотя в целом перемещаться можно => проверяем
+        // возможность без AVSEEK_FLAG_BACKWARD
+        // Теоретически возможна и обратная ситуация, когда весь индекс < start_time, и
+        // только SeekCall(AVSEEK_FLAG_BACKWARD) == true, но реально это будет означать
+        // бесполезность индекса => ошибка
+        // 
+        //if( !SeekCall(ic, ic->start_time, AVSEEK_FLAG_BACKWARD) )
+        if( !SeekCall(ic, ic->start_time, 0) )
         {
             // проверка индекса/возможности перемещения
             err_str = _("Can't seek through the file");
@@ -405,14 +482,7 @@ bool OpenInfo(FFData& ffi, const char* fname, std::string& err_str)
         //dec->error_concealment = FF_EC_GUESS_MVS | FF_EC_DEBLOCK;
         //dec->error_recognition = FF_ER_CAREFUL;
     
-        uint tag = FFCodecID2Tag(dec->codec_id);
-        std::string tag_str = boost::format("0x%1$04x") % tag % bf::stop;
-        unsigned char c0 = GetChar(tag, 0), c8 = GetChar(tag, 8), 
-            c16 = GetChar(tag, 16), c24 = GetChar(tag, 24);
-        if( isprint(c0) && isprint(c8) && isprint(c16) && isprint(c24) )
-            tag_str = boost::format("%1%%2%%3%%4% / %5%") 
-                % c0 % c8 % c16 % c24 % tag_str % bf::stop;
-                            
+        std::string tag_str = CodecID2Str(dec->codec_id);
         // AVCodec - это одиночка, а AVCodecContext - состояние для него
         // в соответ. потоке контейнера 
         AVCodec* codec = avcodec_find_decoder(dec->codec_id);
@@ -422,7 +492,25 @@ bool OpenInfo(FFData& ffi, const char* fname, std::string& err_str)
             return false;
         }
 
+        // :TRICKY: вся полезна инфо о дорожке, включая размеры видео, реально парсится 
+        // в av_find_stream_info(), а в avcodec_open() - кодек только привязывается к
+        // контексту
+        // Более того, в версиях libavcodec 53.4.x-53.9.x есть ошибка, портящая размеры
+        // для h.264, в процессе вызова avcodec_open() (попало в Ubuntu Oneiric)
+        // (см. b47904d..2214191, черт бы тебя побрал, Felipe Contreras, лезть не в свое дело!)
+        Point sz(VideoSize(dec));
+        if( sz.IsNull() )
+        {
+            err_str = "Video has null size";
+            return false;
+        }
+        ffi.vidSz = sz;
+
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(54,0,0)
+        if( IsFFError(avcodec_open2(dec, codec, 0)) )
+#else
         if( IsFFError(avcodec_open(dec, codec)) )
+#endif
         {
             err_str = boost::format("Can't open codec: %1%") % tag_str % bf::stop;
             return false;
@@ -432,20 +520,21 @@ bool OpenInfo(FFData& ffi, const char* fname, std::string& err_str)
         // * декодер настроен
         //
         ffi.videoIdx = video_idx;
-    
-        Point sz(VideoSize(dec));
-        if( sz.IsNull() )
-        {
-            err_str = "Video has null size";
-            return false;
-        }
-        ffi.vidSz = sz;
 
         res = true;
     }
     return res;
 }
 
+bool OpenInfo(FFData& ffi, const char* fname, std::string& err_str)
+{
+    FFDiagnosis diag;
+    bool res = OpenInfo(ffi, fname, diag);
+
+    err_str.swap(diag.errStr);
+    return res;
+}
+
 FFInfo::FFInfo() {}
 
 FFInfo::FFInfo(const std::string& fname)
@@ -460,12 +549,6 @@ FFInfo::~FFInfo()
     CloseInfo(*this);
 }
 
-bool CanOpenAsFFmpegVideo(const char* fname, std::string& err_str)
-{
-    FFInfo ffi;
-    return OpenInfo(ffi, fname, err_str);
-}
-
 bool FFViewer::Open(const char* fname, std::string& err_str)
 {
     // * закрываем открытое ранее
@@ -499,7 +582,7 @@ bool FFViewer::Open(const char* fname, std::string& err_str)
         ASSERT( rgbCnvCtx );
     
         Point dst_sz(sz);
-        uint8_t* rgbBuf = (uint8_t*)av_malloc(avpicture_get_size(dst_pf, dst_sz.x, dst_sz.y) * sizeof(uint8_t));
+        rgbBuf = (uint8_t*)av_malloc(avpicture_get_size(dst_pf, dst_sz.x, dst_sz.y) * sizeof(uint8_t));
         avcodec_get_frame_defaults(&rgbFrame); // не помешает
         avpicture_fill((AVPicture*)&rgbFrame, rgbBuf, dst_pf, dst_sz.x, dst_sz.y);
     }
@@ -523,9 +606,30 @@ static double TS2Time(int64_t ts, FFViewer& ffv)
     return tm;
 }
 
+// с версий больше чем 0.6 используем skip_frame вместо hurry_up,
+// потому что все равно отрубят последний
+#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52,65,0)
+#define USE_SKIP_FRAME
+#endif
+
 static bool IsInHurry(AVCodecContext* dec)
 {
+#ifdef USE_SKIP_FRAME
+    return dec->skip_frame >= AVDISCARD_NONREF;
+#else
     return dec->hurry_up != 0;
+#endif
+}
+
+static void SetHurryUp(AVCodecContext* dec, bool is_on)
+{
+#ifdef USE_SKIP_FRAME
+    UNUSED_VAR(dec);
+    UNUSED_VAR(is_on);
+#else
+    // как признак (хоть и устаревший)
+    dec->hurry_up =  is_on ? 1 : 0;
+#endif
 }
 
 struct HurryModeEnabler
@@ -534,8 +638,7 @@ struct HurryModeEnabler
 
     HurryModeEnabler(AVCodecContext* dec_): dec(dec_)
     {
-        // как признак (хоть и устаревший)
-        dec->hurry_up = 1;
+        SetHurryUp(dec, true);
         // Прирост скорости (h264): 
         // - AVDISCARD_NONREF: 2x
         // - AVDISCARD_BIDIR: для h264 (и других современных кодеков?) разница в скорости 
@@ -555,7 +658,7 @@ struct HurryModeEnabler
     }
    ~HurryModeEnabler()
     {
-        dec->hurry_up = 0;
+        SetHurryUp(dec, false);
         dec->skip_frame = AVDISCARD_DEFAULT;
         //dec->skip_idct = AVDISCARD_DEFAULT;
         //dec->skip_loop_filter = AVDISCARD_DEFAULT;
@@ -611,8 +714,10 @@ static void DoVideoDecode(FFViewer& ffv, int& got_picture, AVPacket* pkt)
     int av_res = avcodec_decode_video(GetVideoCtx(ffv), &picture, &got_picture, buf, buf_sz);
 #endif
     if( av_res < 0 )
+    {    
         // ничего не требуется делать в случае ошибок
         LOG_WRN << "Error while decoding frame!" << io::endl;
+    }
 }
 
 static void ResetCurPTS(FFViewer& ffv)
@@ -662,7 +767,7 @@ static bool DoDecode(FFViewer& ffv)
         }
         av_free_packet(&pkt);
     }
-    else if( av_res == AVERROR_EOF ) // для mpegts также -EIO приходит
+    else if( av_res == (int)AVERROR_EOF ) // для mpegts также -EIO приходит
     {
         // остатки в декодере забираем
         DoVideoDecode(ffv, got_picture, 0);
@@ -817,7 +922,9 @@ static bool DoSeek(FFViewer& ffv, int64_t ts, bool is_byte_seek)
     // * перемещение
     // если перемещение не прошло (индекс частично поломан), то полагаем, что
     // состояние прежнее (обнуление не требуется)
-    bool res = SeekCall(ffv.iCtx, ts, is_byte_seek);
+    int flags = is_byte_seek ? AVSEEK_FLAG_BYTE
+        : AVSEEK_FLAG_BACKWARD; // чтоб раньше времени пришли
+    bool res = SeekCall(ffv.iCtx, ts, flags);
     if( res )
     {
         // * обнуляем
@@ -879,22 +986,34 @@ std::string PrefContents(const char* fname)
     return user_opts;
 }
 
+bool ReadPref(const char* name, RPData& rp)
+{
+    bool& is_read = rp.isRead;
+    
+    bool res = !is_read;
+    if( res )
+    {
+        is_read = true;
+        rp.val = PrefContents(name);
+    }
+    return res;
+}
+
 static bool SeekSetTime(FFViewer& ffv, double time)
 {
     bool is_begin = false;
     double start_time = StartTime(ffv);
     for( int i=0; i<4 && !is_begin; i++ )
     {
-        static double seek_shift  = 0.;
-        static bool is_shift_read = false;
-        if( !is_shift_read )
+        static double seek_shift = 0.;
+        static RPData ss_rp;
+        if( ReadPref("av_seek_shift", ss_rp) )
         {
-            is_shift_read = true;
-            std::string str = PrefContents("av_seek_shift");
-            if ( !str.empty() )
+            std::string str = ss_rp.val;
+            if( !str.empty() )
             {
                 double val;
-                if ( Str::GetType<double>(val, str.c_str()) )
+                if( Str::GetType<double>(val, str.c_str()) )
                 {
                     const double max_val = 15;
                     if ( (val < -max_val) || (val > max_val) )
@@ -922,15 +1041,15 @@ static bool SeekSetTime(FFViewer& ffv, double time)
 
     if( is_begin )
     {
-        if( !TimeSeek(ffv, start_time, time) && CanByteSeek(ffv.iCtx) )
+        bool seek_ok = TimeSeek(ffv, start_time, time);
+        if( !seek_ok && CanByteSeek(ffv.iCtx) )
             // тогда переходим в начало файла
-            DoSeek(ffv, ffv.iCtx->data_offset, true);
-        //TimeSeek(ffv, start_time, time);
+            seek_ok = DoSeek(ffv, ffv.iCtx->data_offset, true);
 
         // некоторое видео глючит в начале (Hellboy), из-за чего
         // последовательный доступ выполняется с перескоками -
         // явно ставим пред. кадр
-        if( IsCurPTS(ffv) )
+        if( seek_ok && IsCurPTS(ffv) )
             // :KLUDGE: -1 уже занят, поэтому -0.5
             // (система работает, пока start_time не бывает отрицательным)
             ffv.prevPTS = start_time - 0.5;
@@ -1004,3 +1123,149 @@ RefPtr<Gdk::Pixbuf> GetRawFrame(double time, FFViewer& ffv)
     return res_pix;
 }
 
+namespace DVD {
+
+struct VobCtx
+{
+   int64_t  curPos;
+   VobFile  vFile;
+   
+   VobCtx(VobPtr vob, dvd_reader_t* dvd): curPos(0), vFile(vob, dvd) {}
+};
+    
+// только для передачи параметров для протокола bmdvob
+static VobPtr BmdVob;    
+static dvd_reader_t* BmdDVD = 0;    
+
+static int VobOpen(URLContext *h, const char *filename, int flags)
+{
+    // параметры передаем внешним образом
+    ASSERT( strcmp(filename, "bmdvob:") == 0 );
+#ifdef AVFORMAT_54
+    ASSERT( flags == AVIO_FLAG_READ );
+#else
+    ASSERT( flags == URL_RDONLY );
+#endif
+    
+    VobCtx* vc = new VobCtx(BmdVob, BmdDVD);
+    h->priv_data = (void*) vc;
+    return 0;
+}
+
+static int64_t DoSeek(VobCtx* vc, int64_t n_pos, int whence)
+{
+    int64_t cnt  = Size(vc->vFile);
+    int64_t& pos = vc->curPos;
+    switch( whence )
+    {
+    case SEEK_SET:
+        pos = n_pos;
+        break;
+    case SEEK_CUR:
+        pos += n_pos;
+        break;
+    case SEEK_END:
+        pos = cnt + n_pos;
+        break;
+    default:
+        ASSERT(0);
+    }
+    pos = std::max(int64_t(0), pos);
+    pos = std::min(pos, cnt);
+    return pos;
+}
+
+static int VobRead(URLContext *h, unsigned char *buf, int sz)
+{
+    VobCtx* vc = (VobCtx*)h->priv_data;
+    int ret = -1;
+    if( sz > 0 )
+    {
+        int64_t old_pos = vc->curPos;
+        ret = DoSeek(vc, sz, SEEK_CUR) - old_pos;
+        
+        try
+        {
+            ReadVob((char*)buf, ret, vc->vFile, old_pos);
+        }
+        catch( const std::exception& exc )
+        {
+            LOG_INF << "VobRead fail: " << exc.what() << io::endl;
+            ret = -1;
+        }
+    }
+    return ret;
+}
+
+static int64_t VobSeek(URLContext *h, int64_t n_pos, int whence)
+{
+    VobCtx* vc = (VobCtx*)h->priv_data;
+    
+    int64_t ret = 0;
+    if( whence == AVSEEK_SIZE )
+        ret = Size(vc->vFile);
+    else
+        ret = DoSeek(vc, n_pos, whence);
+    
+    return ret;
+}
+
+static int VobClose(URLContext *h)
+{
+    delete (VobCtx*)h->priv_data;
+    return 0;
+}
+    
+static void RegisterVobProt()
+{
+    static bool is_init = false;
+    if( !is_init )
+    {
+        is_init = true;
+        
+        // так как постоянно добавляют атрибуты, то лучше так
+        //static URLProtocol bmdvob_up = {
+        //    "bmdvob",
+        //    VobOpen,
+        //    VobRead,
+        //    0, //file_write,
+        //    VobSeek,
+        //    VobClose,
+        //    0,0,0,0
+        //};
+        static URLProtocol bmdvob_up;
+        int sz = sizeof(URLProtocol);
+        memset(&bmdvob_up, 0, sz);
+
+        bmdvob_up.name = "bmdvob";
+        bmdvob_up.url_open  = VobOpen;
+        bmdvob_up.url_read  = VobRead;
+        bmdvob_up.url_seek  = VobSeek;
+        bmdvob_up.url_close = VobClose;
+                
+#ifdef AVFORMAT_54
+        ffurl_register_protocol(&bmdvob_up, sz);
+#elif LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(52, 69, 0)
+        // :TRICKY: с некоторого момента сделали его deprecated,
+        // а вместо него ffurl_register_protocol(), только в публичном заголовке его нет
+        av_register_protocol2(&bmdvob_up, sz);
+#else
+        av_register_protocol(&bmdvob_up);
+#endif
+    }
+}
+   
+bool OpenVob(FFViewer& ffv, VobPtr vob, dvd_reader_t* dvd, std::string& err_str)
+{
+    RegisterVobProt();
+    
+    BmdVob = vob;
+    BmdDVD = dvd;
+    bool res = ffv.Open("bmdvob:", err_str);
+    BmdVob = 0;
+    BmdDVD = 0;
+    return res;
+}
+
+} // namespace DVD
+
