@@ -24,6 +24,7 @@
 
 #include <mlib/patterns.h>
 #include <string>
+#include <map>
 
 enum PlayAuthoring
 {
@@ -35,8 +36,12 @@ enum PlayAuthoring
 struct Preferences: public Singleton<Preferences>
 {
              bool  isPAL;   // PAL vs NTSC
+             bool  remMyTVChoice; // помнить выбор по умолчанию isPAL при старте
+
     PlayAuthoring  player;
       std::string  authorPath;
+             bool  showSrcFileBrowser; // добавление медиа по-старому
+              int  maxCPUWorkload;
 
             Preferences() { Init(); }
 
@@ -49,6 +54,8 @@ inline Preferences& Prefs()
 }
 
 void LoadPrefs();
+void SavePrefs();
+
 void ShowPrefs(Gtk::Window* win = 0);
 
 //
@@ -63,17 +70,29 @@ inline void TryDefaultAuthorPath(Gtk::FileChooser& fc)
 }
 
 //
-// Восстановление размеров с прошлого запуска
+// Автонастройки с прошлого запуска: восстановление размеров и др.
 // 
 
+struct FCState
+{
+    std::string  lastDir;
+            int  lastFilter;
+
+    FCState(const std::string& ld = std::string(), int lf = 0): lastDir(ld), lastFilter(lf) {}
+};
+
+typedef std::map<std::string, FCState> FCMap;
+
 struct UnnamedPreferences: public Singleton<UnnamedPreferences>
 {
     bool   isLoaded; // настройки были загружены, а не по умолчанию
 
     Point  appSz; // размеры и положение приложения
     Point  appPos; 
-      int  fbWdh; // ширина File Browser
-      int  mdBrw1Wdh; // ширина Media Browser на Sources
+      int  srcBrw1Wdh; // ширина первого браузера (Media Browser, File Browser)
+      int  srcBrw2Wdh;  // ширина Media Browser на Sources
+    FCMap  fcMap; // последние открытые папки
+      int  ihsNum; // номер хостинга изображений в ihslist.lst
 
         UnnamedPreferences() { Init(); }
   void  Init();
@@ -86,5 +105,22 @@ void SetUpdatePos(Gtk::HPaned& hpaned, int& saved_pos);
 
 std::string PreferencesPath(const char* fname);
 
+int MaxCPUWorkload();
+
+struct RPData
+{
+          bool  isRead;
+   std::string  val;
+   
+   RPData(): isRead(false) {}
+};
+
+std::string PrefContents(const char* fname);
+// чтение однострочных настроек с кэшированием
+bool ReadPref(const char* name, RPData& rp);
+bool PrefToBool(const std::string& str);
+
+void GoUrl(const gchar* url);
+
 #endif // #ifndef __MGUI_PREFS_H__
 
