@@ -43,16 +43,16 @@
 
 #include <strings.h> // strcasecmp()
 
-namespace Project
-{
-
-static std::string MarkError(const std::string& val, bool not_error)
+std::string MarkError(const std::string& val, bool not_error)
 {
     if( not_error )
         return val;
     return "<span foreground=\"red\">" + val + "</span>";
 }
 
+namespace Project
+{
+
 static std::string MarkBoolError(bool val, bool not_error)
 {
     return MarkError(std::string(val ? _("yes") : _("no")), not_error);
@@ -172,24 +172,28 @@ void CheckVideoFormat(ErrorDesc& ed, const Mpeg::SequenceData& vid, bool is_ntsc
 
 } // namespace 
 
-bool IsVideoDVDCompliant(const char* fname, std::string& err_string, bool& is_mpeg2)
+//bool IsVideoDVDCompliant(const char* fname, std::string& err_string, bool& is_mpeg2)
+bool IsVideoDVDCompliant(const char* fname, Mpeg2Info& add_inf)
 {
     Mpeg::PlayerData pd;
     io::stream& strm     = pd.srcStrm;
     Mpeg::MediaInfo& inf = pd.mInf;
 
     bool res = false;
+    std::string& err_string = add_inf.errStr;
+    bool& video_check = add_inf.videoCheck;
+    
     if( !Mpeg::GetInfo(pd, fname) )
         err_string = inf.ErrorReason();
     else
     {
-        is_mpeg2 = true; // дальше не проверяем тип - это точно видео
+        add_inf.isMpeg2 = true; // дальше не проверяем тип - это точно видео
         Mpeg::SequenceData& vid = inf.vidSeq;
         ErrorDesc ed;
         // * видео
         bool is_ntsc = !IsPALProject();
         CheckVideoFormat(ed, vid, is_ntsc);
-        bool video_check = ed.res;
+        video_check = ed.res;
 
         // * мультиплексирование в формате DVD
         bool is_dvd_mux   = false;
@@ -294,6 +298,11 @@ inline bool CaseIEqual(const std::string &s1, const std::string &s2)
     return strcasecmp(s1.c_str(), s2.c_str()) == 0;
 }
 
+bool GetPicDimensions(const char* fname, Point& sz)
+{
+    return gdk_pixbuf_get_file_info(fname, &sz.x, &sz.y);
+}
+
 // определить тип файла и создать по нему соответствующее медиа
 StorageItem CreateMedia(const char* fname, std::string& err_string)
 {
@@ -316,17 +325,25 @@ StorageItem CreateMedia(const char* fname, std::string& err_string)
 
     // Сейчас открытие с помощью CanOpenAsFFmpegVideo()/FFData считаем 
     // эталоном в плане принятия материалов; в частности:
-    // - ошибка возвращается с темы, если не смогли открыть как изображение
+    // - ошибка возвращается с темы, если не смогли открыть как видео
     // - тема не должна открывать аудио или картинки, чтоб был правильный тип
     // Вроде как GdkPixbuf умел открывать видео (его первый кадр) и ради этого
     // был поставлен доп. заслон в виде must_be_video (сейчас не повторяется на
     // mpeg/m2v/..); если повторится, то улучшаем проверку (проверяем по кодеку 
     // у FFmpeg,- для картинок он "image2" не должен быть)
-    int wdh, hgt;
-    if( CanOpenAsFFmpegVideo(fname, err_string) )
+    FFInfo ffi;
+    FFDiagnosis ff_diag;
+    if( OpenInfo(ffi, fname, ff_diag) )
         md = new VideoMD;
-    else if( gdk_pixbuf_get_file_info(fname, &wdh, &hgt) )
-        md = new StillImageMD;
+    else
+    {
+        bool must_be_video = ff_diag.isElemStream; // всякие .m2v
+        err_string = ff_diag.errStr;
+
+        Point sz;
+        if( !must_be_video && GetPicDimensions(fname, sz) )
+            md = new StillImageMD;
+    }
 
     if( md )
         md->MakeByPath(fname);
@@ -370,23 +387,22 @@ StorageItem CreateMedia(const char* fname, std::string& err_string)
 
 // pth - куда вставлять; по выходу pth равен позиции вставленного
 // insert_after - вставить после pth, по возможности
-bool TryAddMedia(const char* fname, Gtk::TreePath& pth, std::string& err_str, 
-                 bool insert_after = true)
+StorageItem TryAddMedia(const char* fname, Gtk::TreePath& pth, std::string& err_str, 
+                        bool insert_after)
 {
-    bool res = false;
-    if( StorageItem md = CreateMedia(fname, err_str) )
+    StorageItem md;
+    if( md = CreateMedia(fname, err_str) )
     {
-        res = true;
         LOG_INF << "Insert Media!" << io::endl;
 
-        RefPtr<MediaStore> ms = GetAStores().mdStore;
+        RefPtr<MediaStore> ms = GetMediaStore();
         Gtk::TreeIter itr = InsertByPos(ms, pth, insert_after);
         PublishMedia(itr, ms, md);
         InvokeOnInsert(md);
 
         pth = ms->get_path(itr);
     }
-    return res;
+    return md;
 }
 
 // desc - метка происхождения, добавления
@@ -394,7 +410,8 @@ void TryAddMediaQuiet(const std::string& fname, const std::string& desc)
 {
     std::string err_str;
     Gtk::TreePath pth;
-    if( !TryAddMedia(fname.c_str(), pth, err_str) )
+    bool res = TryAddMedia(fname.c_str(), pth, err_str);
+    if( !res )
     {    
         LOG_ERR << "TryAddMediaQuiet error (" << desc << "): " << err_str << io::endl;
     }
@@ -419,6 +436,27 @@ static void AddMediaError(const std::string& msg_str, const std::string& desc_st
 #endif
 }
 
+StorageItem CheckExists(const fs::path& pth, RefPtr<MediaStore> ms)
+{
+    StorageItem res;
+    for( MediaStore::iterator itr = ms->children().begin(), end = ms->children().end();
+         itr != end; ++itr )
+    {
+        StorageItem si = GetAsStorage(ms->GetMedia(itr));
+        if( fs::equivalent(pth, si->GetPath()) )
+        {
+            res = si;
+            break;
+        }
+    }
+    return res;
+}
+
+void OneMediaError(const fs::path& err_pth, const std::string& desc)
+{
+    AddMediaError(BF_("Can't add file \"%1%\".") % err_pth.leaf() % bf::stop, desc);
+}
+
 void TryAddMedias(const Str::List& paths, MediaBrowser& brw,
                   Gtk::TreePath& brw_pth, bool insert_after)
 {
@@ -472,23 +510,28 @@ void TryAddMedias(const Str::List& paths, MediaBrowser& brw,
         std::string ConvertPathToUtf8(const std::string& path);
         fs::path pth = ConvertPathToUtf8(fpath);
 
-        bool is_exist = false;
         // * проверяем, есть ли такой уже
-        RefPtr<MediaStore> ms = brw.GetMediaStore();
-        for( MediaStore::iterator itr = ms->children().begin(), end = ms->children().end();
-             itr != end; ++itr )
+        //bool is_exist = false;
+        //RefPtr<MediaStore> ms = brw.GetMediaStore();
+        //for( MediaStore::iterator itr = ms->children().begin(), end = ms->children().end();
+        //     itr != end; ++itr )
+        //{
+        //    StorageItem si = GetAsStorage(ms->GetMedia(itr));
+        //    if( fs::equivalent(pth, si->GetPath()) )
+        //    {
+        //        // только переходим к нему
+        //        brw.get_selection()->select(GetBrowserPath(si));
+        //        is_exist = true;
+        //        break;
+        //    }
+        //}
+        //if( is_exist )
+        //    continue;
+        if( StorageItem si = CheckExists(pth, brw.GetMediaStore()) )
         {
-            StorageItem si = GetAsStorage(ms->GetMedia(itr));
-            if( fs::equivalent(pth, si->GetPath()) )
-            {
-                // только переходим к нему
-                brw.get_selection()->select(GetBrowserPath(si));
-                is_exist = true;
-                break;
-            }
-        }
-        if( is_exist )
+            brw.get_selection()->select(GetBrowserPath(si));
             continue;
+        }
 
         bool res = TryAddMedia(fpath.c_str(), brw_pth, err_str, insert_after);
         if( res )
@@ -540,7 +583,7 @@ void TryAddMedias(const Str::List& paths, MediaBrowser& brw,
         // кол-во заполнителей 
         //boost::format frmt(ngettext("Can't add file \"%1%\".", "Can't add files:", err_cnt));
         if( one_error )
-            AddMediaError(BF_("Can't add file \"%1%\".") % err_pth.leaf() % bf::stop, desc);
+            OneMediaError(err_pth, desc);
         else
             AddMediaError(_("Can't add files:"), desc);
     }
@@ -549,20 +592,8 @@ void TryAddMedias(const Str::List& paths, MediaBrowser& brw,
         GoToPos(brw, goto_path);
 }
 
-Str::List GetFilenames(Gtk::FileChooser& fc)
+void MediaBrowserAdd(MediaBrowser& brw, Str::List& paths)
 {
-    // я знаю, что это жутко неэффективно (2+1 копирования), но не актуально
-    typedef Glib::SListHandle<Glib::ustring> UList;
-    UList ulist = fc.get_filenames();
-
-    Str::List list;
-    std::copy(ulist.begin(), ulist.end(), std::back_inserter(list));
-    return list;
-}
-
-void MediaBrowserAdd(MediaBrowser& brw, Gtk::FileChooser& fc)
-{
-    Str::List paths(GetFilenames(fc));
     // * куда
     Gtk::TreePath brw_pth = GetCursor(brw);
     ValidateMediaInsertionPos(brw_pth);
