@@ -170,7 +170,7 @@ namespace Project
 
 void PackMediasWindow(Gtk::Container& contr, RefPtr<MediaStore> ms, MediasWindowPacker pack_fnr)
 {
-    MediaBrowser& brw = NewManaged<MediaBrowser>(ms);
+    MediaBrowser& brw = NewManaged<MediaBrowser>(ms, true);
     pack_fnr(contr, brw);
 }
 
@@ -240,30 +240,33 @@ Rect GetAllocation(Gtk::Widget& wdg)
 }
 
 static bool EraseTabLineOnExpose(Gtk::Notebook& nbook, GdkEventExpose* event, bool erase_down,
-				 Gtk::Notebook* book_tabs)
+                                 Gtk::Notebook* book_tabs)
 {
     Rect plc = GetAllocation(nbook);
-    if( !plc.IsNull() )
+    if ( !plc.IsNull() )
     {
-        plc.lft += 1;
-        if( erase_down )
-    	{
-    	    plc.rgt -= 1;
-    	    plc.top = plc.btm-1;
-    	}
+        plc.lft += 1; // для Clearlooks границу не стирать (хотя для умолчальной удобнее без уменьшения)
+        // стираем 2 ряда пикселов с каждой стороны:
+        // - clearlooks etc - толщина верхней 2 пиксела, нижней - 1
+        // - умолчальная - наоборот
+        if ( erase_down )
+        {
+            plc.rgt -= 1;
+            plc.top = plc.btm-2;
+        }
         else
-    	{
+        {
             // стираем верхнюю линию рамки только до окончания верхних табов
-    	    plc.rgt = std::min(plc.rgt, GetAllocation(*book_tabs).rgt)-1;
-    	    plc.btm = plc.top+1;
-    	}
-        if( plc.Intersects(MakeRect(event->area)) )
+            plc.rgt = std::min(plc.rgt, GetAllocation(*book_tabs).rgt)-1;
+            plc.btm = plc.top+2;
+        }
+        if ( plc.Intersects(MakeRect(event->area)) )
         {
             Cairo::RefPtr<Cairo::Context> cr = nbook.get_window()->create_cairo_context();
-            cr->set_line_width(erase_down ? 1.0 : 2.0);
+            cr->set_line_width(2.0);
             CR::SetColor(cr, GetBGColor(nbook));
 
-            double y = plc.top + (erase_down ? H_P : 1);
+            double y = plc.top + 1;
             cr->move_to(plc.lft, y);
             cr->line_to(plc.rgt, y);
             cr->stroke();
@@ -630,7 +633,8 @@ void UpdateDVDSize()
 
         if( sz > typ_sz )
             sz_str = "<span color=\"red\">" + sz_str + "</span>";
-        sz_str += " of ";
+        // for translators: 100MB of 4.3GB
+        sz_str += std::string(" ") + _("of") + " ";
     }
     Application().SB().szLbl.set_markup(sz_str);
 }
@@ -651,6 +655,12 @@ void MuxAddStreams(const std::string& src_fname)
 
 //////////////////////////////////////////////////
 
+static void OnMenuSR(Gtk::Requisition* req)
+{
+    if( req->width )
+        req->width += 30;
+}
+
 ConstructorApp::ConstructorApp(): askSaveOnExit(true), isProjectChanged(false)
 {
     Add(win, vBox);
@@ -731,9 +741,17 @@ ConstructorApp::ConstructorApp(): askSaveOnExit(true), isProjectChanged(false)
         }
 
         // разведем меню и закладки на небольшое расстояние
-        main_bar.show_all();
-        Gtk::Requisition req = main_bar.size_request();
-        main_bar.set_size_request(req.width+30, req.height);
+        //main_bar.show_all();
+        //Gtk::Requisition req = main_bar.size_request();
+        //main_bar.set_size_request(req.width+30, req.height);
+        // 02.09.2011:
+        // - с проявлением активности Ubuntu в области глобального меню пришлось
+        //   по-другому увеличивать размер меню (более простой способ - добавить
+        //   например, пустое меню, но граница между ними будет видна для некоторых тем);
+        // - вообще, глобальное меню (апплет "Indicator Applet AppMenu") реализовано за счет сокрытия 
+        //   первого локального GtkMenuBar, причем информация о изменении статуса локального приходит 
+        //   через DBus; поэтому заранее про размер не рассчитать (ворачивается 0)
+        main_bar.signal_size_request().connect(&OnMenuSR);
 
         // *
         // Высота вкладок (по крайней мере в Clearlooks) чуть больше, чем у меню (Menubar);
@@ -870,6 +888,9 @@ ActionFunctor BuildConstructor(ConstructorApp& app, const std::string& prj_file_
     app.win.set_default_size(up.appSz.x, up.appSz.y);
     if( up.isLoaded )
         app.win.move(up.appPos.x, up.appPos.y);
+    else
+        // в первый раз - по центру
+        app.win.set_position(Gtk::WIN_POS_CENTER);
     app.win.signal_configure_event().connect(&UpdateAppSizes, false);
     
     // *
@@ -889,8 +910,39 @@ ActionFunctor BuildConstructor(ConstructorApp& app, const std::string& prj_file_
     return after_fnr;
 }
 
+static bool DestroyWin(ptr::shared<Gtk::Window>& swin)
+{
+    swin.reset();
+    return false;
+}
+
 void RunConstructor(const std::string& prj_file_name, bool ask_save_on_exit)
 {
+    // заставка
+    //Gtk::Window splash_win;
+    ptr::shared<Gtk::Window> swin(new Gtk::Window);
+    Gtk::Window& splash_win = *swin;
+    
+    // :KLUDGE: в идеале правильней явно вызывать gdk_window_set_back_pixmap(),
+    // но иметь дело с устаревающим GdkPixmap (в 3.0 заменен на cairo_surface)
+    // не хочется (тот же depth вычислять),- через стиль проще
+    splash_win.Gtk::Widget::set_name("SplashWin");
+    // :TRICKY: IteratePendingEvents() недостаточно для прорисовки splash'а,-
+    // вроде как нужно устанавливать фон окна вместо добавления Gtk::Image;
+    // при этом нужно явно размеры окна указать - сделаем это неявно, добавлением
+    // того самого Gtk::Image (лень переписывать код)
+    Add(splash_win, NewManaged<Gtk::Image>(DataDirImage("splash-bmd.png")));
+    
+    splash_win.set_position(Gtk::WIN_POS_CENTER);
+    splash_win.set_resizable(false);
+    splash_win.set_decorated(false);
+    splash_win.show_all();
+    // :KLUDGE: если вызвать до show_all(), то не сработает под win32
+    // (gdk_window_set_keep_above() не вызывается)
+    splash_win.set_keep_above(true);
+    // принудить (GTK/OS/...) выполнить всю текущую работу => показать окно заставки быстрей
+    IteratePendingEvents();
+
     DBCleanup db_cleanup(false);
     // *
     InitI18n();
@@ -907,13 +959,22 @@ void RunConstructor(const std::string& prj_file_name, bool ask_save_on_exit)
     
         std::list<RefPtr<Gdk::Pixbuf> > pix_lst;
         static const fs::directory_iterator end_itr;
-        for( fs::directory_iterator itr(DataDirPath("icons"));
+        for( fs::directory_iterator itr(DataDirPath("bmd-icons"));
             itr != end_itr; ++itr )
             pix_lst.push_back(Gdk::Pixbuf::create_from_file(itr->string()));
         Gtk::Window::set_default_icon_list(pix_lst);
     
         ActionFunctor after_fnr = BuildConstructor(app, prj_file_name);
-        RunWindow(app.win);
+        //RunWindow(app.win);
+        app.win.show_all();
+        // отложим закрытие заставки на 0.5 секунд
+        //swin.reset();
+        Timer().Connect(bb::bind(&DestroyWin, b::ref(swin)), 500);
+
+        if( !Prefs().remMyTVChoice )
+            OnNewProject(app, true);
+        
+        Gtk::Main::run(app.win);
         after_fnr();
     }
     // сохраняем настройки после закрытия (=> обновления информации) окон
