@@ -37,6 +37,7 @@
 #include <mgui/key.h>
 #include <mgui/win_utils.h>
 #include <mgui/execution.h> // PipeOutput()
+#include <mgui/prefs.h>
 
 #include <mgui/editor/toolbar.h>
 
@@ -211,29 +212,37 @@ static bool OnOBButtonPress(ObjectBrowser& brw, const RightButtonFunctor& fnr, G
     return true;
 }
 
+static bool& IsVideoOK(RTCache& rtc)
+{
+    return rtc.asd.videoOK;
+}
+
 RTCache& GetRTC(VideoItem vi)
 {
     RTCache& rtc = vi->GetData<RTCache>();
     if( !rtc.isCalced )
     {
         const std::string& fname = GetFilename(*vi);
-        bool is_mpeg2;
-        std::string err_string;
-        rtc.reqTrans = !IsVideoDVDCompliant(fname.c_str(), err_string, is_mpeg2);
+        Mpeg2Info inf;
+        rtc.reqTrans = !IsVideoDVDCompliant(fname.c_str(), inf);
+        IsVideoOK(rtc) = inf.videoCheck;
+        // если без транскодирования => videoOK == true
+        ASSERT_RTL( rtc.reqTrans || (!rtc.reqTrans && IsVideoOK(rtc)) );
 
         FFInfo ffi(GetFilename(*vi).c_str());
         rtc.duration = Duration(ffi);
         rtc.vidSz    = ffi.vidSz;
-        rtc.dar      = DAspectRatio(ffi);
+        rtc.asd.dar  = DAspectRatio(ffi);
         // расчет числа аудио
         AVFormatContext* ic = ffi.iCtx;
         // :TODO: отрефакторить цикл
-        rtc.audioNum = 0;
+        int& a_cnt = rtc.asd.audioNum;
+        a_cnt = 0;
         for( int i=0; i < (int)ic->nb_streams; i++ )
         {
             AVCodecContext* avctx = ic->streams[i]->codec;
-            if( avctx->codec_type == CODEC_TYPE_AUDIO )
-                rtc.audioNum++;
+            if( avctx->codec_type == AVMEDIA_TYPE_AUDIO )
+                a_cnt++;
         }
 
         rtc.isCalced = true;
@@ -247,6 +256,11 @@ bool RequireTranscoding(VideoItem vi)
     return GetRTC(vi).reqTrans;
 }
 
+bool RequireVideoTC(VideoItem vi)
+{
+    return !IsVideoOK(GetRTC(vi));
+}
+
 static void AppendNamedValue(Gtk::VBox& vbox, RefPtr<Gtk::SizeGroup> sg, const char* name, 
                              const std::string& value)
 {
@@ -353,15 +367,16 @@ static DVDDims Dimensions(BitrateControls& bc)
     return Index2Dimension(bc.cmbDims.get_active_row_number());
 }
 
-int OutAudioNum(int i_anum)
+int OutAudioNum(const AutoSrcData& asd)
 {
+    int i_anum = asd.audioNum;
     ASSERT( i_anum >= 0 );
     return std::min(i_anum, 8);
 }
 
-io::pos CalcTransSize(RTCache& rtc, int vrate)
+static io::pos _CalcTransSize(RTCache& rtc, int vrate)
 {
-    return FFmpegSizeForDVD(rtc.duration, vrate, OutAudioNum(rtc.audioNum));
+    return FFmpegSizeForDVD(rtc.duration, vrate, OutAudioNum(rtc.asd));
 }
 
 static void OnBitrateControlsChanged(BitrateControls& bc, bool dims_changed)
@@ -370,7 +385,7 @@ static void OnBitrateControlsChanged(BitrateControls& bc, bool dims_changed)
         SetAutoBitrate(bc, Dimensions(bc));
 
     // расчет размера результата
-    gint64 fsize = CalcTransSize(bc.rtc, bc.vRate.get_value());
+    gint64 fsize = _CalcTransSize(bc.rtc, bc.vRate.get_value());
     bc.szLbl.set_text(ShortSizeString(fsize));
 }
 
@@ -399,7 +414,25 @@ static void SetTransData(VideoItem vi, DVDDims dd, int vrate)
     td.vRate = vrate;
 }
 
-static void RunBitrateCalc(VideoItem vi, Gtk::Dialog& dlg)
+// :TODO: использовать fe::range<> нельзя, потому что он не хранит копию временного ListHandle_Path,
+// а только его итераторы (которые по выходу из функции становятся недействительными)
+Gtk::TreeSelection::ListHandle_Path AllSelected(Gtk::TreeView& brw)
+{
+    return brw.get_selection()->get_selected_rows();
+}
+
+std::string& CustomFFOpts(VideoItem vi)
+{
+    return vi->transDat.ctmFFOpt;
+}
+
+static void UpdateTransSettings(VideoItem vi, BitrateControls& bc, const std::string& ctm_ff_opt)
+{
+    SetTransData(vi, Dimensions(bc), bc.vRate.get_value());
+    CustomFFOpts(vi) = ctm_ff_opt;
+}
+
+static void RunBitrateCalc(VideoItem vi, Gtk::Dialog& dlg, ObjectBrowser& brw)
 {
     dlg.set_title(boost::format("%1% (%2%)") % _("Bitrate Calculator") % vi->mdName % bf::stop);
 
@@ -459,15 +492,52 @@ static void RunBitrateCalc(VideoItem vi, Gtk::Dialog& dlg)
         // он хоть адекватен
         //AppendNamedValue(vbox, SMCLN_("Video bitrate"),
         //                 boost::format("%1% %2%") % (GetVideoCtx(ffv)->bit_rate/1000) % kbps_txt % bf::stop);
+        Point& dar = rtc.asd.dar;
         AppendNamedValue(info_box, sg, SMCLN_("Display aspect ratio"), 
-                         boost::format("%1% : %2%") % rtc.dar.x % rtc.dar.y % bf::stop);
-        AppendNamedValue(info_box, sg, SMCLN_("Number of audio streams"), Int2Str(rtc.audioNum));
+                         boost::format("%1% : %2%") % dar.x % dar.y % bf::stop);
+        AppendNamedValue(info_box, sg, SMCLN_("Number of audio streams"), Int2Str(rtc.asd.audioNum));
         AppendNamedValue(info_box, sg, SMCLN_("File size"), ShortSizeString(PhisSize(fname.c_str())));
     }
+    
+    std::string ctm_ff_opt = CustomFFOpts(vi);
+    Gtk::Expander& expdr = NewManaged<Gtk::Expander>(_("Additional _ffmpeg options"), true);
+    expdr.set_expanded(ctm_ff_opt.size());
+    SetTip(expdr, _("Examples: \"-top 0\", \"-deinterlace\". See FFmpeg documentation for more options."));
+
+    Gtk::ComboBoxEntryText& custom_cmb = Add(expdr, NewManaged<Gtk::ComboBoxEntryText>());
+    // :TODO: можно сохранять в авто-настройках
+    static Str::List history_lst;
+    boost_foreach( std::string str, history_lst )
+        custom_cmb.append_text(str);
+    Gtk::Entry& custom_ent = *custom_cmb.get_entry();
+    custom_ent.set_text(ctm_ff_opt);
+    PackStart(vbox, expdr);
 
     if( CompleteAndRunOk(dlg) )
     {
-        SetTransData(vi, Dimensions(bc), bc.vRate.get_value());
+        ctm_ff_opt = custom_ent.get_text();
+        // обновляем текущее видео (под курсором) и все выделенные
+        // :KLUDGE: вообще, поведение при выделении объектов + правой кнопки мыши
+        // отличается от нормального (см. например Nautilus), поэтому в выделенном
+        // может не быть текущего видео => необходим лишний вызов
+        UpdateTransSettings(vi, bc, ctm_ff_opt);
+        boost_foreach( Gtk::TreePath pth, AllSelected(brw) )
+        {
+            VideoItem vi = IsVideo(GetMedia(brw.GetObjectStore(), pth));
+            if( IsTransVideo(vi, true) )
+                UpdateTransSettings(vi, bc, ctm_ff_opt);
+        }
+
+        Str::List::iterator it = std::find(history_lst.begin(), history_lst.end(), ctm_ff_opt);
+        if( it != history_lst.end() )
+            std::swap(history_lst[0], *it);
+        else
+        {
+            history_lst.insert(history_lst.begin(), ctm_ff_opt);
+            if( history_lst.size() > 10 )
+                history_lst.resize(10);
+        }
+        
         UpdateDVDSize();
     }
 }
@@ -475,11 +545,11 @@ static void RunBitrateCalc(VideoItem vi, Gtk::Dialog& dlg)
 static void ShowDVDCompliantStatus(VideoItem vi)
 {
     std::string err_string(_("Reason For Transcoding"));
-    std::string desc_string;
-    bool is_mpeg2 = false;
-    if( IsVideoDVDCompliant(GetFilename(*vi).c_str(), desc_string, is_mpeg2) )
+    Mpeg2Info inf;
+    std::string& desc_string = inf.errStr;
+    if( IsVideoDVDCompliant(GetFilename(*vi).c_str(), inf) )
         err_string = _("The video is DVD compliant.");
-    else if( !is_mpeg2 )
+    else if( !inf.isMpeg2 )
         desc_string = _("The video is not MPEG2.");
 
     MessageBox(err_string, Gtk::MESSAGE_INFO, Gtk::BUTTONS_OK, desc_string);
@@ -490,31 +560,53 @@ double Duration(VideoItem vi)
     return GetRTC(vi).duration;
 }
 
+static io::pos PhisSize(VideoItem vi)
+{
+    return PhisSize(GetFilename(*vi).c_str());
+}
+
+io::pos CalcTransSize(VideoItem vi, int vrate)
+{
+    RTCache& rtc = GetRTC(vi);
+    io::pos res = _CalcTransSize(rtc, vrate);
+    if( IsVideoOK(rtc) )
+        // перемикширование
+        // :KLUDGE: +1% взят "с потолка", как страховка от того, что
+        // оригинальный битрейт аудио будет существенно меньше нашего TRANS_AUDIO_BITRATE
+        res = PhisSize(vi) * 1.01;
+    return res;
+}
+
 static SizeStat ProjectStatEx(bool fixed_part)
 {
     SizeStat ss;
-    io::pos& sz = ss.prjSum;
+    io::pos& vsz = ss.videoSum;
     io::pos& tr_sz = ss.transSum;
 
     boost_foreach( VideoItem vi, AllVideos() )
     {
         if( RequireTranscoding(vi) )
         {
-            RTCache& rtc = GetRTC(vi);
             int vrate    = fixed_part ? 0 : GetRealTransData(vi).vRate ;
-            io::pos v_sz = CalcTransSize(rtc, vrate);
+            io::pos v_sz = CalcTransSize(vi, vrate);
 
             tr_sz += v_sz;
-            sz    += v_sz;
+            vsz   += v_sz;
         }
         else
-            sz += PhisSize(GetFilename(*vi).c_str());
+            vsz += PhisSize(vi);
     }
-    sz += Author::MenusSize();
-
+    //sz += Author::MenusSize();
+    ss.menuSum = Author::MenusSize();
+    
     return ss;
 }
 
+io::pos PrjSum(const SizeStat& ss)
+{
+    return ss.videoSum + ss.menuSum;
+}
+
 SizeStat ProjectStat()
 {
     return ProjectStatEx(false);
@@ -522,7 +614,8 @@ SizeStat ProjectStat()
 
 io::pos ProjectSizeSum(bool fixed_part)
 {
-    return ProjectStatEx(fixed_part).prjSum;
+    //return ProjectStatEx(fixed_part).prjSum;
+    return PrjSum(ProjectStatEx(fixed_part));
 }
 
 DVDDims GetRealTD(VideoItem vi)
@@ -548,12 +641,12 @@ static void AdjustDiscUsage()
     io::pos work_sz = (dvd_sz - ProjectSizeSum(true)) / TRANS_OVER_ASSURANCE;
 
     double total_weight = 0.;
-    boost_foreach( VideoItem vi, AllTransVideos() )
+    boost_foreach( VideoItem vi, AllVTCVideos() )
         total_weight += RelTransWeight(vi);
     ASSERT( total_weight > 0. );
 
     bool is_overflow = false;
-    boost_foreach( VideoItem vi, AllTransVideos() )
+    boost_foreach( VideoItem vi, AllVTCVideos() )
     {
         // в kbit/s
         int vrate = work_sz * RelTransWeight(vi)/(total_weight * 125 * Duration(vi));
@@ -620,7 +713,12 @@ static void SetSubtitles(VideoItem vi, Gtk::Dialog& dlg)
 
     DialogVBox& vbox = AddHIGedVBox(dlg);
     SetFilename(s_btn, dat.pth);
-    AppendWithLabel(vbox, s_btn, SMCLN_("Select subtitles"));
+    Gtk::HBox& hbox = PackCompositeWdg(s_btn);
+    // CANCEL DELETE DISCARD NO REMOVE STOP CLEAR CLOSE
+    // по поводу очистки GtkFileChooserButton: https://bugzilla.gnome.org/show_bug.cgi?id=612235
+    PackCompositeWdgButton(hbox, Gtk::Stock::CLEAR, bb::bind(&Gtk::FileChooser::unselect_all, &s_btn), _("Unselect subtitles"));
+    AppendWithLabel(vbox, hbox, SMCLN_("Select subtitles"));
+
     ds_btn.set_active(dat.defShow);
     PackStart(vbox, ds_btn);
     // взято у DeVeDe с минимальными изменениями
@@ -628,7 +726,7 @@ static void SetSubtitles(VideoItem vi, Gtk::Dialog& dlg)
     // при помощи enca надеемся на совпадение алиасов кодировок
     io::stream strm(DataDirPath("copy-n-paste/codepages.lst").c_str(), iof::in);
     std::string enc_str;
-    for( int i=0; std::getline(strm, enc_str), strm; i++ )
+    for( int i=0; std::getline(strm, enc_str); i++ )
     {
         enc_cmb.append_text(enc_str);
         if( dat.encoding == enc_str )
@@ -651,9 +749,87 @@ DialogParams SubtitlesDialog(VideoItem vi, Gtk::Widget* par_wdg)
     return DialogParams(_("Add Subtitles"), bb::bind(&SetSubtitles, vi, _1), 400, par_wdg);
 }
 
+void RenderField(Gtk::CellRenderer* rndr, const Gtk::TreeModel::iterator& iter,
+                 const I2TFunctor& fnr)
+{
+    static_cast<Gtk::CellRendererText*>(rndr)->property_text() = fnr(iter);
+}
+
+void SetTextRendererFnr(Gtk::TreeView::Column& name_cln, Gtk::CellRendererText& rndr, 
+                        const I2TFunctor& fnr)
+{
+    name_cln.set_cell_data_func(rndr, bb::bind(&RenderField, _1, _2, fnr));
+}
+
+static std::string Iter2MI(RefPtr<ObjectStore> os, const RFFunctor& fnr,
+                           const Gtk::TreeModel::iterator& iter)
+{
+    return fnr(os->GetMedia(iter));
+}
+
+void SetRendererFnr(Gtk::TreeView::Column& name_cln, Gtk::CellRendererText& rndr, 
+                    RefPtr<ObjectStore> os, const RFFunctor& fnr)
+{
+    SetTextRendererFnr(name_cln, rndr, bb::bind(&Iter2MI, os, fnr, _1));
+}
+
+// :TRICKY: без свойства "editable" редактирования не будет вообще, с ним - 
+// будет и по левой кнопке мыши (а это не уровень); "классический" вариант отключения (nautilus) -
+// включать "editable" перед редактированием, и выключать позже, но в двух местах (на ok и cancel), что
+// некруто; а вот способ подмены start_editing не обладает этим недостатком (хотя игра на грани)
+static bool& CanRename()
+{
+    static bool can_rename = false;
+    return can_rename;
+}
+
+class MyCellRendererText: public Gtk::CellRendererText
+{
+    typedef Gtk::CellRendererText MyParent;
+    protected:
+    virtual Gtk::CellEditable* start_editing_vfunc(GdkEvent* event, Gtk::Widget& widget, const Glib::ustring& path, 
+                                                   const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, Gtk::CellRendererState flags)
+    {
+        Gtk::CellEditable* res = 0;
+        if( CanRename() )
+            res = MyParent::start_editing_vfunc(event, widget, path, background_area, cell_area, flags);
+        return res;
+    }
+};
+
+Gtk::CellRendererText& MakeNameRenderer()
+{
+    return NewManaged<MyCellRendererText>();
+}
+
+static void RenameCurItem(ObjectBrowser& brw)
+{
+    // находим колонку 
+    const char* name = _("Name");
+    Gtk::TreeViewColumn* n_cln = 0;
+    boost_foreach( Gtk::TreeViewColumn* cln, brw.get_columns() )
+        if( cln->get_title() == name )
+        {
+            n_cln = cln;
+            break;
+        }
+    ASSERT_RTL( n_cln );
+    
+    CanRename() = true;
+    brw.set_cursor(GetCursor(brw), *n_cln, true);
+    CanRename() = false;
+}
+
+void AppendRenameAction(Gtk::Menu& mn, ObjectBrowser& brw)
+{
+    AddEnabledItem(mn, _("Rename"), bb::bind(&RenameCurItem, b::ref(brw)), true);
+}
+
 static void OnMBButtonPress(ObjectBrowser& brw, MediaItem mi, GdkEventButton* event)
 {
-    Gtk::Menu& mn  = NewPopupMenu(); 
+    Gtk::Menu& mn = NewPopupMenu(); 
+    AppendRenameAction(mn, brw);
+    AppendSeparator(mn);
     Gtk::MenuItem& ea_itm = AppendMI(mn, NewManaged<Gtk::MenuItem>(_("End Action")));
     // только видео
     VideoItem vi = IsVideo(mi);
@@ -661,11 +837,10 @@ static void OnMBButtonPress(ObjectBrowser& brw, MediaItem mi, GdkEventButton* ev
         ea_itm.set_submenu(EndActionMenuBld(vi->PAction(), boost::function_identity,
                                             VideoAddConstantChoice).Create());
 
-    AppendSeparator(mn);
-    bool tr_enabled = IsTransVideo(vi);
+    bool tr_enabled = IsTransVideo(vi, true);
     AddEnabledItem(mn, _("Adjust Bitrate to Fit to Disc"), &AdjustDiscUsage, tr_enabled);
     // калькулятор
-    AddDialogItem(mn, DialogParams(_("Bitrate Calculator"), bb::bind(&RunBitrateCalc, vi, _1), 
+    AddDialogItem(mn, DialogParams(_("Bitrate Calculator"), bb::bind(&RunBitrateCalc, vi, _1, b::ref(brw)), 
                                    350, &brw), tr_enabled);
     AddEnabledItem(mn, _("Reason For Transcoding"), bb::bind(&ShowDVDCompliantStatus, vi), vi);
     AppendSeparator(mn);
@@ -680,13 +855,29 @@ void SetOnRightButton(ObjectBrowser& brw, const RightButtonFunctor& fnr)
     sig::connect(brw.signal_button_press_event(), bb::bind(&OnOBButtonPress, boost::ref(brw), fnr, _1), false);
 }
 
-MediaBrowser::MediaBrowser(RefPtr<MediaStore> a_lst)
+struct ImageRTCache
 {
-    set_model(a_lst);
-    BuildStructure();
+     bool  isCalced;
+    Point  sz;
 
-    SetupURIDrop(*this, bb::bind(&OnURIsDrop, boost::ref(*this), _1, _2));
-    SetOnRightButton(*this, bb::bind(&OnMBButtonPress, _1, _2, _3));
+    ImageRTCache(): isCalced(false) {}
+};
+
+Point GetStillImageDimensions(StorageItem still_img)
+{
+    // win-backend очень тормозной у gdk_pixbuf_get_file_info() - кэширование не избежать
+    ImageRTCache& rtc = still_img->GetData<ImageRTCache>();
+    Point& sz = rtc.sz;
+    // :TODO: refactor
+    if( !rtc.isCalced )
+    {
+        rtc.isCalced = true;
+
+        bool true_ = GetPicDimensions(GetFilename(*still_img).c_str(), sz);
+        ASSERT_OR_UNUSED( true_ );
+    }
+
+    return sz;
 }
 
 // Названия типов для i18n
@@ -694,46 +885,71 @@ F_("Video")
 F_("Chapter")
 F_("Still Picture")
 
-void RenderMediaType(Gtk::CellRendererText* rndr, MediaItem mi)
+static std::string RenderMediaType(MediaItem mi, bool show_info)
 {
-    rndr->property_text() = gettext(mi->TypeString().c_str());
+    //return gettext(mi->TypeString().c_str());
+    std::string info = gettext(mi->TypeString().c_str());
+    if( show_info )
+    {
+        if( VideoItem vi = IsVideo(mi) )
+        {
+            RTCache& rtc = GetRTC(vi);
+            Point& dar = rtc.asd.dar;
+            info = boost::format("%1%, %2%, %3%x%4%, %5%:%6%") % info % Mpeg::SecToHMS(rtc.duration, true) 
+                % rtc.vidSz.x % rtc.vidSz.y % dar.x % dar.y % bf::stop;
+        }
+        else if( ChapterItem ci = IsChapter(mi) )
+            info = BF_("Chapter at %1%") % Mpeg::SecToHMS(ci->chpTime, true) % bf::stop;
+        else if( StorageItem sii = IsStillImage(mi) )
+        {
+            Point sz = GetStillImageDimensions(sii);
+            info = boost::format("%1%, %2%x%3%") % info % sz.x % sz.y % bf::stop;
+        }
+    }
+    return info;
 }
 
-void MediaBrowser::BuildStructure()
+Gtk::CellRendererText& AppendNameColumn(Gtk::TreeView& tv, const Gtk::TreeModelColumn<RefPtr<Gdk::Pixbuf> >& thumbnail_cln,
+                                        bool set_resizable, RefPtr<ObjectStore> os)
 {
-    RefPtr<MediaStore> ms = GetMediaStore();
+    Gtk::TreeView::Column& name_cln = NewManaged<Gtk::TreeView::Column>(_("Name"));
+    name_cln.set_resizable(set_resizable);
+
+    name_cln.pack_start(thumbnail_cln, false);
+
+    // имя
+    Gtk::CellRendererText& rndr = MakeNameRenderer();
+    SetupNameRenderer(name_cln, rndr, os);
+
+    tv.append_column(name_cln);
+    return rndr;
+}
+
+MediaBrowser::MediaBrowser(RefPtr<MediaStore> ms, bool show_info)
+{
+    set_model(ms);
     const MediaStore::TrackFields& trk_fields = MediaStore::Fields();
 
     SetupBrowser(*this, trk_fields.media.index(), true);
 
     // 1 миниатюра + имя
-    {
-        Gtk::TreeView::Column& name_cln = *Gtk::manage( new Gtk::TreeView::Column(_("Name")) );
-        // ширину колонки можно менять
-        name_cln.set_resizable(true);
-        
-        name_cln.pack_start(trk_fields.thumbnail, false);
-        
-        // 2 имя
-        Gtk::CellRendererText& rndr = *Gtk::manage( new Gtk::CellRendererText() );
-        //name_cln.set_renderer(rndr, trk_fields.title);
-        SetupNameRenderer(name_cln, rndr, ms);
+    AppendNameColumn(*this, trk_fields.thumbnail, true, ms);
 
-        append_column(name_cln);
-    }
-
-    // 3 тип
+    // 2 тип
     {
-        Gtk::TreeView::Column& cln  = *Gtk::manage( new Gtk::TreeView::Column(_("Type")) );
+        Gtk::TreeView::Column& cln  = NewManaged<Gtk::TreeView::Column>(show_info ? _("Information") : _("Type"));
         Gtk::CellRendererText& rndr = *Gtk::manage( new Gtk::CellRendererText() );
 
         cln.pack_start(rndr, false);
         // не используем данных,- вычисляем на лету
         //cln.set_renderer(rndr, trk_fields.title);
-        SetRendererFnr(cln, rndr, ms, &RenderMediaType);
+        SetRendererFnr(cln, rndr, ms, bb::bind(&RenderMediaType, _1, show_info));
 
         append_column(cln);
     }
+
+    SetupURIDrop(*this, bb::bind(&OnURIsDrop, boost::ref(*this), _1, _2));
+    SetOnRightButton(*this, bb::bind(&OnMBButtonPress, _1, _2, _3));
 }
 
 void ExecuteForMedia(MediaBrowser& mb, MediaActionFnr fnr)
@@ -806,6 +1022,16 @@ static void SetDefaultButtonOnEveryMap(Gtk::Button& btn)
     btn.signal_map().connect(bb::bind(&SetDefaultButton, boost::ref(btn)));
 }
 
+const char* AddFilesDialogTitle()
+{
+    return _("Add Media Files (Use Ctrl Button for Multiple Selection)");
+}
+
+const char* AddFilesTip()
+{
+    return !Prefs().showSrcFileBrowser ? AddFilesDialogTitle() : _("Add Media from File Browser") ;
+}
+
 void PackMediaBrowserAll(Gtk::Container& contr, MediaBrowser& brw, ActionFunctor add_media_fnr, 
                          ActionFunctor remove_media_fnr, ActionFunctor edit_media_fnr)
 {
@@ -819,8 +1045,7 @@ void PackMediaBrowserAll(Gtk::Container& contr, MediaBrowser& brw, ActionFunctor
         Gtk::HButtonBox& hbox = CreateMListButtonBox();
         vbox->pack_start(hbox, Gtk::PACK_SHRINK);
         {
-            Gtk::Button* add_btn = CreateButtonWithIcon("", Gtk::Stock::ADD, 
-                                                        _("Add Media from File Browser"));
+            Gtk::Button* add_btn = CreateButtonWithIcon("", Gtk::Stock::ADD, AddFilesTip());
             hbox.pack_start(*add_btn);
             //bbox.pack_start(*add_btn);
             add_btn->signal_clicked().connect(add_media_fnr);
