@@ -27,7 +27,10 @@
 #include <mgui/sdk/packing.h>
 #include <mgui/sdk/window.h>
 #include <mgui/sdk/widget.h> // ForAllWidgets()
+#include <mgui/prefs.h> // fcMap
 #include <mgui/gettext.h>
+#include <mgui/timeline/mviewer.h> // FileFilterList
+#include <mgui/sdk/win32filesel.h> // CallWinFileDialog
 
 #include <mbase/project/table.h> // Project::ConvertPathToUtf8()
 #include <mbase/resources.h>
@@ -194,7 +197,7 @@ Gtk::Frame& PackWidgetInFrame(Gtk::Widget& wdg, Gtk::ShadowType st, const std::s
 
 // символы вроде "&<>#" перед передачей GMarkupParser должны быть заменены
 // на escape-последовательности
-static std::string QuoteForGMarkupParser(const std::string& str)
+std::string QuoteForGMarkupParser(const std::string& str)
 {
     return Glib::Markup::escape_text(str).raw();
 }
@@ -253,21 +256,37 @@ void SetTip(Gtk::Widget& wdg, const char* tooltip)
 Gtk::Button* CreateButtonWithIcon(const char* label, const Gtk::BuiltinStockID& stock_id,
                                   const char* tooltip, Gtk::BuiltinIconSize icon_sz)
 {
-    Gtk::Button& btn = NewManaged<Gtk::Button>(label);
     Gtk::Image& img  = NewManaged<Gtk::Image>(stock_id, icon_sz);
-    btn.set_image(img);
-    // убираем влияние настройки gtk-button-images после включения в кнопку
-    //img.set_visible();
-    img.property_visible() = true;
+    Gtk::Button* btn = 0;
+    if( label && *label )
+    {
+        btn = &NewManaged<Gtk::Button>(label);
+        btn->set_image(img);
+        // убираем влияние настройки gtk-button-images после включения в кнопку
+        //img.set_visible();
+        img.property_visible() = true;
+    }
+    else
+    {
+        btn = &NewManaged<Gtk::Button>();
+        btn->add(img);
+    }
 
-    SetTip(btn, tooltip);
-    return &btn;
+    SetTip(*btn, tooltip);
+    return btn;
 }
 
 void AddCancelDoButtons(Gtk::Dialog& dialog, Gtk::BuiltinStockID do_id)
 {
+    // в стандартных диалогах вроде Открыть/Сохранить порядок
+    // кнопок определяется настройкой gtk-alternative-button-order = [1|0]
+#ifdef _WIN32
+    dialog.add_button(do_id, Gtk::RESPONSE_OK);
+    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+#else
     dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
     dialog.add_button(do_id, Gtk::RESPONSE_OK);
+#endif
     dialog.set_default_response(Gtk::RESPONSE_OK);
 }
 
@@ -298,26 +317,99 @@ void BuildChooserDialog(Gtk::FileChooserDialog& dialog, bool is_open, Gtk::Widge
     AddCancelDoButtons(dialog, is_open ? Gtk::Stock::OPEN : Gtk::Stock::SAVE );
 }
 
+bool RunFileDialog(const char* title, bool open_file, Str::List& chosen_paths, 
+                   Gtk::Widget& for_wdg, const FileFilterList& pat_lst,
+                   bool multiple_choice, const FCDFunctor& fnr)
+{
+    bool res;
+#ifdef _WIN32
+    if( !fnr )
+        // используем нативный диалог, по возможности
+        // :TODO: соответственно, кастомизировать диалоги пока невозможно
+        res = CallWinFileDialog(title, open_file, chosen_paths, &for_wdg, pat_lst, multiple_choice);
+    else
+#endif
+    {
+        Gtk::FileChooserDialog dialog(title, open_file ? Gtk::FILE_CHOOSER_ACTION_OPEN : Gtk::FILE_CHOOSER_ACTION_SAVE);
+        
+        FCMap& fc_map = UnnamedPrefs().fcMap;
+        // :TODO: (субъективно) чтобы локализация title не мешала, необходимо
+        // gettext() делать здесь, а при вызове,- C_()
+        FCMap::iterator itr = fc_map.find(title);
+        int last_flt = 0;
+        if( itr != fc_map.end() )
+        {
+            FCState& fcs = itr->second;
+            dialog.set_current_folder(fcs.lastDir);
+            last_flt = fcs.lastFilter;
+        }
+
+        if( chosen_paths.size() )
+            dialog.set_current_name(chosen_paths[0]);
+
+        BuildChooserDialog(dialog, open_file, for_wdg);
+
+        int idx = 0;
+        boost_foreach( const FileFilter& ff, pat_lst )
+        {
+            // :TRICKY: если на стеке создать, то повторного доступа к С++-оболочкам
+            // потом не будет
+            Gtk::FileFilter& gff = NewManaged<Gtk::FileFilter>();
+            gff.set_name(ff.title);
+            boost_foreach( const std::string& glob_pat, ff.extPatLst )
+            {
+                //gff.add_pattern(ext);
+                AddGlobFilter(gff, glob_pat);
+            }
+
+            dialog.add_filter(gff);
+            if( last_flt && (idx == last_flt) )
+                dialog.set_filter(gff);
+            idx++;
+        }
+
+        if( fnr )
+            fnr(dialog);
+        dialog.set_select_multiple(multiple_choice);
+
+        std::string fname;
+        for( ; res = Gtk::RESPONSE_OK == dialog.run(), res; )
+        {
+            fname = dialog.get_filename();
+            if( !open_file && CheckKeepOrigin(fname) )
+                continue;
+
+            chosen_paths = GetFilenames(dialog);
+            break;
+        }
+        
+        // сохраняем последние настройки
+        if( res )
+        {
+            // приходиться перебором находить
+            last_flt = 0;
+            Gtk::FileFilter* cur_ff = dialog.get_filter();
+            boost_foreach( const Gtk::FileFilter* ff, dialog.list_filters() )
+            {
+                if( cur_ff == ff )
+                    break;
+                last_flt++;
+            }
+            fc_map[title] = FCState(dialog.get_current_folder(), last_flt);
+        }
+    }
+
+    return res;
+}
+
 bool ChooseFileSaveTo(std::string& fname, const std::string& title, Gtk::Widget& for_wdg,
                       const FCDFunctor& fnr)
 {
-    Gtk::FileChooserDialog dialog(title, Gtk::FILE_CHOOSER_ACTION_SAVE);
-    BuildChooserDialog(dialog, false, for_wdg);
-
-    dialog.set_current_name(fname);
-    if( fnr )
-        fnr(dialog);
-
-    bool res;
-    for( ; res = Gtk::RESPONSE_OK == dialog.run(), res; )
-    {
-        fname = dialog.get_filename();
-        if( CheckKeepOrigin(fname) )
-            continue;
-
-        break;
-    }
-
+    Str::List paths;
+    paths.push_back(fname);
+    bool res = RunFileDialog(title.c_str(), false, paths, for_wdg, FileFilterList(), false, fnr);
+    if( res )
+        fname = paths[0];
     return res;
 }
 
@@ -405,7 +497,7 @@ void CompleteDialog(Gtk::Dialog& dlg, bool close_style)
     if( close_style )
         dlg.add_button(Gtk::Stock::CLOSE, Gtk::RESPONSE_CLOSE);
     else
-        AddCancelDoButtons(dlg, Gtk::Stock::OK);
+        AddCancelOKButtons(dlg);
     dlg.get_vbox()->show_all();
 }
 
